def get_next_output_file_name(file_extension='ly'):
    '''Get next output file name like ``6223.ly``.

    Return string.
    '''
    from abjad.tools import iotools
    from abjad.tools.iotools._warn_almost_full import _warn_almost_full

    assert file_extension.isalpha() and 0 < len(file_extension) < 4

    last_output = iotools.get_last_output_file_name()
    if last_output is None:
        next_number = 0
        next_output_file_name = '0000.{}'.format(file_extension)
    else:
        last_number = int(last_output.split('.')[0])
        next_number = last_number + 1
        next_output_file_name = '{next_number:04d}.{file_extension}'.format(
            next_number=next_number, file_extension=file_extension)
    if 9000 < next_number:
        _warn_almost_full(last_number)
    return next_output_file_name



import abc
from abjad.tools.abctools.AbjadObject import AbjadObject


class HarmonicObject(AbjadObject):
    '''..versionadded:: 2.0

    Harmonic object base class.
    '''

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta

    __slots__ = ()

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self):
        pass



from abjad.tools import mathtools


def partition_sequence_by_weights_at_most(sequence, weights, cyclic=False, overhang=False):
    r'''.. versionadded:: 1.1

    Partition `sequence` by `weights` at most.

        >>> sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]

    Example 1. Partition sequence once by weights at most without overhang::

        >>> sequencetools.partition_sequence_by_weights_at_most(
        ...     sequence, [10, 4], cyclic=False, overhang=False)
        [[3, 3, 3], [3]]

    Example 2. Partition sequence once by weights at most with overhang::

        >>> sequencetools.partition_sequence_by_weights_at_most(
        ...     sequence, [10, 4], cyclic=False, overhang=True)
        [[3, 3, 3], [3], [4, 4, 4, 4, 5, 5]]

    Example 3. Partition sequence cyclically by weights at most without overhang::

        >>> sequencetools.partition_sequence_by_weights_at_most(
        ...     sequence, [10, 5], cyclic=True, overhang=False)
        [[3, 3, 3], [3], [4, 4], [4], [4, 5], [5]]

    Example 4. Partition sequence cyclically by weights at most with overhang::

        >>> sequencetools.partition_sequence_by_weights_at_most(
        ...     sequence, [10, 5], cyclic=True, overhang=True)
        [[3, 3, 3], [3], [4, 4], [4], [4, 5], [5]]

    Return list of sequence objects.
    '''
    if not cyclic:
        return _partition_sequence_once_by_weights_at_most(sequence, weights, overhang=overhang)
    else:
        return _partition_sequence_cyclically_by_weights_at_most(sequence, weights, overhang=overhang)


def _partition_sequence_once_by_weights_at_most(sequence, weights, overhang=False):

    l_copy = sequence[:]
    result = []
    current_part = []

    for target_weight in weights:
        while True:
            try:
                x = l_copy.pop(0)
            except IndexError:
                raise PartitionError('too few elements in sequence.')
            current_weight = mathtools.weight(current_part)
            candidate_weight = current_weight + mathtools.weight([x])
            if candidate_weight < target_weight:
                current_part.append(x)
            elif candidate_weight == target_weight:
                current_part.append(x)
                result.append(current_part)
                current_part = []
                break
            elif target_weight < candidate_weight:
                if current_part:
                    result.append(current_part)
                    current_part = []
                    l_copy.insert(0, x)
                    break
                else:
                    raise PartitionError('Elements in sequence too big.')
            else:
                raise ValueError('candidate and target weights must compare.')

    if overhang:
        left_over = current_part + l_copy
        if left_over:
            result.append(left_over)

    return result


def _partition_sequence_cyclically_by_weights_at_most(sequence, weights, overhang=False):

    result = []
    current_part = []
    current_target_weight_index = 0
    current_target_weight = weights[current_target_weight_index]
    l_copy = sequence[:]

    while l_copy:
        current_target_weight = weights[current_target_weight_index % len(weights)]
        x = l_copy.pop(0)
        current_part_weight = mathtools.weight(current_part)
        candidate_part_weight = current_part_weight + mathtools.weight([x])
        if candidate_part_weight < current_target_weight:
            current_part.append(x)
        elif candidate_part_weight == current_target_weight:
            current_part.append(x)
            result.append(current_part)
            current_part = []
            current_target_weight_index += 1
        elif current_target_weight < candidate_part_weight:
            if current_part:
                l_copy.insert(0, x)
                result.append(current_part)
                current_part = []
                current_target_weight_index += 1
            else:
                raise PartitionError('Elements in sequence too big.')
        else:
            raise ValueError('candidate and target rates must compare.')

    if current_part:
        if overhang:
            result.append(current_part)

    return result



from abjad.tools.rhythmmakertools.DivisionIncisedNoteRhythmMaker import DivisionIncisedNoteRhythmMaker


class NoteRhythmMaker(DivisionIncisedNoteRhythmMaker):
    r'''.. versionadded:: 2.8

    Note rhythm-maker::

        >>> maker = rhythmmakertools.NoteRhythmMaker()

    ::

        >>> divisions = [(5, 16), (3, 8)]
        >>> leaf_lists = maker(divisions)
        >>> leaves = sequencetools.flatten_sequence(leaf_lists)

    ::

        >>> measures = measuretools.make_measures_with_full_measure_spacer_skips(divisions)
        >>> staff = Staff(measures)
        >>> measures = measuretools.replace_contents_of_measures_in_expr(staff, leaves)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 5/16
                c'4 ~
                c'16
            }
            {
                \time 3/8
                c'4.
            }
        }

    Usage follows the two-step instantiate-then-call pattern shown here.

    Return rhythm-maker.
    '''

    ### CLASS ATTRIBUTES ###

    _default_positional_input_arguments = ()

    ### INITIALIZER ###

    def __init__(self, decrease_durations_monotonically=True, tie_rests=False):
        DivisionIncisedNoteRhythmMaker.__init__(
            self, [], [0], [], [0], 1,
            decrease_durations_monotonically=decrease_durations_monotonically, tie_rests=tie_rests
            )

    ### SPECIAL METHODS ###

    def __repr__(self):
        return '%s()' % type(self).__name__



def offset_happens_before_timespan_starts(timespan=None, offset=None, hold=False):
    r'''.. versionadded:: 2.11

    Make time relation indicating that `offset` happens before `timespan` starts::

        >>> timerelationtools.offset_happens_before_timespan_starts()
        OffsetTimespanTimeRelation('offset < timespan.start')

    Make time relation indicating that offset ``1/2`` happens before `timespan` starts::

        >>> offset = durationtools.Offset(1, 2)

    ::

        >>> time_relation = timerelationtools.offset_happens_before_timespan_starts(
        ...     offset=offset)

    ::

        >>> z(time_relation)
        timerelationtools.OffsetTimespanTimeRelation(
            'offset < timespan.start',
            offset=durationtools.Offset(1, 2)
            )

    Make time relation indicating that `offset` happens before timespan ``[2, 8)`` starts::

        >>> timespan = timespantools.Timespan(2, 8)

    ::

        >>> time_relation = timerelationtools.offset_happens_before_timespan_starts(
        ...     timespan=timespan)

    ::

        >>> z(time_relation)
        timerelationtools.OffsetTimespanTimeRelation(
            'offset < timespan.start',
            timespan=timespantools.Timespan(
                start_offset=durationtools.Offset(2, 1),
                stop_offset=durationtools.Offset(8, 1)
                )
            )

    Make time relation indicating that offset ``1/2`` happens before 
    timespan ``[2, 8)`` starts::

        >>> time_relation = timerelationtools.offset_happens_before_timespan_starts(
        ...     timespan=timespan, offset=offset, hold=True)

    ::

        >>> z(time_relation)
        timerelationtools.OffsetTimespanTimeRelation(
            'offset < timespan.start',
            timespan=timespantools.Timespan(
                start_offset=durationtools.Offset(2, 1),
                stop_offset=durationtools.Offset(8, 1)
                ),
            offset=durationtools.Offset(1, 2)
            )

    Evaluate time relation indicating that offset ``1/2`` happens before 
    timespan ``[2, 8)`` starts::

        >>> timerelationtools.offset_happens_before_timespan_starts(
        ...     timespan=timespan, offset=offset, hold=False)
        True

    Return time relation or boolean.
    '''
    from abjad.tools import timerelationtools

    time_relation = timerelationtools.OffsetTimespanTimeRelation(
        'offset < timespan.start',
        timespan=timespan, offset=offset)

    if time_relation.is_fully_loaded and not hold:
        return time_relation()
    else:
        return time_relation



from abjad.tools import quantizationtools
from abjad.tools.quantizationtools.JobHandler._Job import _Job


def test_SerialJobHandler___call___01():

    jobs = [_Job(x) for x in range(1, 11)]
    job_handler = quantizationtools.SerialJobHandler()
    finished_jobs = job_handler(jobs)



from abjad.tools import iterationtools
from abjad.tools import measuretools
from abjad.tools.wellformednesstools.Check import Check


class MisfilledMeasureCheck(Check):
    '''Check that time signature duration equals measure contents duration for every measure.
    '''

    def _run(self, expr):
        violators = []
        total, bad = 0, 0
        for measure in iterationtools.iterate_components_in_expr(expr, measuretools.Measure):
            if measure.is_misfilled:
                violators.append(measure)
                bad += 1
            total += 1
        return violators, total



from abjad import *
import py


def test_Measure_append_01():
    '''Time signature does not automatically adjust.
    '''

    measure = Measure((3, 4), "c' d' e'")
    measure.append('r')

    assert measure.is_overfull
    assert py.test.raises(Exception, 'f(measure)')


def test_Measure_append_02():
    '''Time signature adjusts automatically.
    '''

    measure = Measure((3, 4), "c' d' e'")
    measure.automatically_adjust_time_signature = True
    measure.append('r')

    assert not measure.is_misfilled
    assert measure.lilypond_format == "{\n\t\\time 4/4\n\tc'4\n\td'4\n\te'4\n\tr4\n}"



import os
from abjad.tools import configurationtools


def ly(target=-1):
    r'''Open the last LilyPond output file in text editor::

        >>> iotools.ly() # doctest: +SKIP

    ::

        % Abjad revision 2162
        % 2009-05-31 14:29

        \version "2.12.2"
        \include "english.ly"
        \include "/Path/to/abjad/trunk/abjad/cfg/abjad.scm"

        {
            c'4
        }

    Open the next-to-last LilyPond output file in text editor::

        >>> iotools.ly(-2) # doctest: +SKIP

    Exit text editor in the usual way.

    Return none.
    '''
    from abjad import ABJCFG
    from abjad.tools import iotools

    ABJADOUTPUT = ABJCFG['abjad_output']
    text_editor = configurationtools.get_text_editor()
    if isinstance(target, int) and target < 0:
        last_lilypond = iotools.get_last_output_file_name()
        if last_lilypond:
            last_number = last_lilypond.replace('.ly', '')
            last_number = last_number.replace('.pdf', '')
            last_number = last_number.replace('.midi', '')
            last_number = last_number.replace('.mid', '')
            target_number = int(last_number) + (target + 1)
            target_str = '%04d' % target_number
            target_ly = os.path.join(ABJADOUTPUT, target_str + '.ly')
        else:
            print 'Target LilyPond input file does not exist.'
    elif isinstance(target, int) and 0 <= target:
        target_str = '%04d' % target
        target_ly = os.path.join(ABJADOUTPUT, target_str + '.ly')
    elif isinstance(target, str):
        target_ly = os.path.join(ABJADOUTPUT, target)
    else:
        raise ValueError('can not get target LilyPond input from {}.'.format(target))

    if os.stat(target_ly):
        command = '{} {}'.format(text_editor, target_ly)
        # TODO: how do we get rid of this last tricky call to os.system()?
        #spawn_subprocess(command)
        os.system(command)
    else:
        print 'Target LilyPond input file {} does not exist.'.format(target_ly)



def get_context_marks_attached_to_component(component, klasses=None):
    r'''.. versionadded:: 2.0

    Get context marks attached to `component`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> contexttools.ClefMark('treble')(staff)
        ClefMark('treble')(Staff{4})
        >>> contexttools.DynamicMark('p')(staff[0])
        DynamicMark('p')(c'8)

    ::

        >>> f(staff)
        \new Staff {
            \clef "treble"
            c'8 \p
            d'8
            e'8
            f'8
        }

    ::

        >>> contexttools.get_context_marks_attached_to_component(staff[0])
        (DynamicMark('p')(c'8),)

    Return tuple of zero or more context marks.
    '''
    from abjad.tools import contexttools

    if klasses is None:
        klasses = (contexttools.ContextMark,)
 
    result = []
    for mark in component._marks_for_which_component_functions_as_start_component:
        if isinstance(mark, klasses):
            result.append(mark)

    return tuple(result)



from abjad.tools import chordtools


def iterate_chords_in_expr(expr, reverse=False, start=0, stop=None):
    r'''.. versionadded:: 2.10

    Iterate chords forward in `expr`::

        >>> staff = Staff("<e' g' c''>8 a'8 r8 <d' f' b'>8 r2")

    ::

        >>> f(staff)
        \new Staff {
            <e' g' c''>8
            a'8
            r8
            <d' f' b'>8
            r2
        }

    ::

        >>> for chord in iterationtools.iterate_chords_in_expr(staff):
        ...   chord
        Chord("<e' g' c''>8")
        Chord("<d' f' b'>8")

    Iterate chords backward in `expr`::

    ::

        >>> for chord in iterationtools.iterate_chords_in_expr(staff, reverse=True):
        ...   chord
        Chord("<d' f' b'>8")
        Chord("<e' g' c''>8")

    Ignore threads.

    Return generator.
    '''
    from abjad.tools import iterationtools

    return iterationtools.iterate_components_in_expr(
        expr, klass=chordtools.Chord, reverse=reverse, start=start, stop=stop)



from abjad import *


def test_instrumenttools_ContraltoVoice_storage_format_01():

    voice = instrumenttools.ContraltoVoice()

    assert voice.storage_format == 'instrumenttools.ContraltoVoice()'




from abjad.tools import chordtools
from abjad.tools import notetools
from abjad.tools.spannertools.Spanner import Spanner


class GlissandoSpanner(Spanner):
    r'''Abjad glissando spanner::

        >>> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        >>> spannertools.GlissandoSpanner(staff[:])
        GlissandoSpanner(c'8, d'8, e'8, f'8)

    ::

        >>> f(staff)
        \new Staff {
            c'8 \glissando
            d'8 \glissando
            e'8 \glissando
            f'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Format nonlast leaves in spanner with LilyPond glissando command.

    Return glissando spanner.
    '''

    ### INITIALIZER ###

    def __init__(self, components=None):
        Spanner.__init__(self, components)

    ### PRIVATE METHODS ###
    
    def _copy_keyword_args(self, new):
        pass

    def _format_right_of_leaf(self, leaf):
        result = []
        if not self._is_my_last_leaf(leaf) and isinstance(leaf, (chordtools.Chord, notetools.Note)):
            result.append(r'\glissando')
        return result



import abc
from abjad.tools.abctools import AbjadObject


class AttackPointOptimizer(AbjadObject):
    '''Abstract attack-point optimizer class from which concrete attack-point
    optimizer classes inherit.

    Attack-point optimizers may alter the number, order, and individual durations
    of leaves in a tie chain, but may not alter the overall duration of that
    tie chain.

    They effectively "clean up" notation, post-quantization.
    '''

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    @abc.abstractmethod
    def __call__(self, expr):
        raise NotImplemented



from abjad.tools.contexttools.ContextMark import ContextMark


class ClefMark(ContextMark):
    r'''.. versionadded:: 2.0

    Abjad model of a clef::

        >>> staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")

    ::

        >>> contexttools.ClefMark('treble')(staff)
        ClefMark('treble')(Staff{8})

    ::

        >>> clef = contexttools.ClefMark('alto')(staff[1])
        >>> clef = contexttools.ClefMark('bass')(staff[2])
        >>> clef = contexttools.ClefMark('treble^8')(staff[3])
        >>> clef = contexttools.ClefMark('bass_8')(staff[4])
        >>> clef = contexttools.ClefMark('tenor')(staff[5])
        >>> clef = contexttools.ClefMark('bass^15')(staff[6])
        >>> clef = contexttools.ClefMark('percussion')(staff[7])

    ::

        >>> f(staff)
        \new Staff {
            \clef "treble"
            c'8
            \clef "alto"
            d'8
            \clef "bass"
            e'8
            \clef "treble^8"
            f'8
            \clef "bass_8"
            g'8
            \clef "tenor"
            a'8
            \clef "bass^15"
            b'8
            \clef "percussion"
            c''8            
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Clef marks target the staff context by default.
    '''

    ### CLASS ATTRIBUTES ###

    _default_positional_input_arguments = (repr('alto'), )

    _format_slot = 'opening'

    #default_target_context = Staff

    ### INITIALIZER ###

    def __init__(self, arg, target_context=None):
        from abjad.tools.stafftools.Staff import Staff
        ContextMark.__init__(self, target_context=target_context)
        if self.target_context is None:
            self._target_context = Staff
        if isinstance(arg, str):
            self._clef_name = arg
        elif isinstance(arg, type(self)):
            self._clef_name = arg.clef_name
        else:
            raise TypeError('can not init clef from %s.' % arg)

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        return type(self)(self._clef_name, target_context=self.target_context)

    __deepcopy__ = __copy__

    def __eq__(self, arg):
        if isinstance(arg, type(self)):
            return self._clef_name == arg._clef_name
        return False

    ### PRIVATE PROPERTIES ###

    _clef_name_to_middle_c_position = { 'treble': -6, 'alto': 0, 'tenor': 2, 'bass': 6, }

    @property
    def _contents_repr_string(self):
        return repr(self._clef_name)

    ### PUBLIC PROPERTIES ###

    @apply
    def clef_name():
        def fget(self):
            r'''Get clef name::

                >>> clef = contexttools.ClefMark('treble')
                >>> clef.clef_name
                'treble'

            Set clef name::

                >>> clef.clef_name = 'alto'
                >>> clef.clef_name
                'alto'

            Return string.
            '''
            return self._clef_name
        def fset(self, clef_name):
            assert isinstance(clef_name, str)
            self._clef_name = clef_name
        return property(**locals())

    @property
    def lilypond_format(self):
        r'''Read-only LilyPond format of clef:

        ::

            >>> clef = contexttools.ClefMark('treble')
            >>> clef.lilypond_format
            '\\clef "treble"'

        Return string.
        '''
        return r'\clef "%s"' % self._clef_name

    @property
    def middle_c_position(self):
        '''Read-only middle-C position of clef:

        ::

            >>> clef = contexttools.ClefMark('treble')
            >>> clef.middle_c_position
            -6

        Return integer number of stafflines.
        '''
        return self._clef_name_to_middle_c_position[self._clef_name]



from abjad.tools.beamtools.ComplexBeamSpanner import ComplexBeamSpanner
from abjad.tools import durationtools
from abjad.tools import sequencetools


class DuratedComplexBeamSpanner(ComplexBeamSpanner):
    r'''Abjad durated complex beam spanner:

    ::

        >>> staff = Staff("c'16 d'16 e'16 f'16")

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> durations = [Duration(1, 8), Duration(1, 8)]
        >>> beam = beamtools.DuratedComplexBeamSpanner(staff[:], durations, 1)

    ::

        >>> f(staff)
        \new Staff {
            \set stemLeftBeamCount = #0
            \set stemRightBeamCount = #2
            c'16 [
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #1
            d'16
            \set stemLeftBeamCount = #1
            \set stemRightBeamCount = #2
            e'16
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #0
            f'16 ]
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Beam all beamable leaves in spanner explicitly.

    Group leaves in spanner according to `durations`.

    Span leaves between duration groups according to `span`.

    Return durated complex beam spanner.
    '''

    def __init__(self, components=None, durations=None, span=1, lone=False, direction=None):
        ComplexBeamSpanner.__init__(self, components=components, direction=direction)
        self.durations = durations
        self.span = span
        self.lone = lone

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _span_points(self):
        result = []
        if self.durations is not None:
            result.append(self.durations[0])
            for d in self.durations[1:]:
                result.append(result[-1] + d)
        return result

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        ComplexBeamSpanner._copy_keyword_args(self, new)
        new.durations = self.durations[:]
        new.span = self.span
            
    def _format_before_leaf(self, leaf):
        from abjad.tools import beamtools
        result = []
        #if leaf.beam.beamable:
        if beamtools.is_beamable_component(leaf):
            if self._is_exterior_leaf(leaf):
                left, right = self._get_left_right_for_exterior_leaf(leaf)
            # just right of span gap
            elif self._duration_offset_in_me(leaf) in self._span_points and not \
                (self._duration_offset_in_me(leaf) + leaf.prolated_duration in \
                self._span_points):
                assert isinstance(self.span, int)
                left = self.span
                #right = leaf.duration._flags
                right = leaf.written_duration.flag_count
            # just left of span gap
            elif self._duration_offset_in_me(leaf) + leaf.prolated_duration in \
                self._span_points and not self._duration_offset_in_me(leaf) in \
                self._span_points:
                assert isinstance(self.span, int)
                #left = leaf.duration._flags
                left = leaf.written_duration.flag_count
                right = self.span
            else:
                left, right = self._get_left_right_for_interior_leaf(leaf)
            if left is not None:
                result.append(r'\set stemLeftBeamCount = #%s' % left)
            if right is not None:
                result.append(r'\set stemRightBeamCount = #%s' % right)
        return result

    def _fracture_left(self, i):
        self, left, right = ComplexBeamSpanner._fracture_left(self, i)
        weights = [left.prolated_duration, right.prolated_duration]
        assert sum(self.durations) == sum(weights)
        split_durations = sequencetools.split_sequence_by_weights(
            self.durations, weights, cyclic=False, overhang=False)
        left_durations, right_durations = split_durations 
        left._durations = left_durations
        right._durations = right_durations
        return self, left, right

    def _fracture_right(self, i):
        self, left, right = ComplexBeamSpanner._fracture_right(self, i)
        weights = [left.prolated_duration, right.prolated_duration]
        assert sum(self.durations) == sum(weights)
        split_durations = sequencetools.split_sequence_by_weights(
            self.durations, weights, cyclic=False, overhang=False)
        left_durations, right_durations = split_durations 
        left._durations = left_durations
        right._durations = right_durations
        return self, left, right

    def _reverse_components(self):
        ComplexBeamSpanner._reverse_components(self)
        self._durations.reverse()

    ### READ / WRITE PUBLIC PROPERTIES ###

    @apply
    def durations():
        def fget(self):
            '''Get spanner leaf group durations::

                >>> staff = Staff("c'16 d'16 e'16 f'16")
                >>> durations = [Duration(1, 8), Duration(1, 8)]
                >>> beam = beamtools.DuratedComplexBeamSpanner(staff[:], durations)
                >>> beam.durations
                [Duration(1, 8), Duration(1, 8)]

            Set spanner leaf group durations::

                >>> staff = Staff("c'16 d'16 e'16 f'16")
                >>> durations = [Duration(1, 8), Duration(1, 8)]
                >>> beam = beamtools.DuratedComplexBeamSpanner(staff[:], durations)
                >>> beam.durations = [Duration(1, 4)]
                >>> beam.durations
                [Duration(1, 4)]

            Set iterable.
            '''
            return self._durations
        def fset(self, arg):
            if arg is None:
                self._durations = None
            elif isinstance(arg, list):
                for i, d in enumerate(arg):
                    if isinstance(d, tuple):
                        arg[i] = durationtools.Duration(*d)
                    else:
                        arg[i] = durationtools.Duration(d)
                self._durations = arg
            else:
                raise ValueError('durations must be list of Durations, or None.')
        return property(**locals())

    @apply
    def span():
        def fget(self):
            r'''Get top-level beam count::

                >>> staff = Staff("c'16 d'16 e'16 f'16")
                >>> durations = [Duration(1, 8), Duration(1, 8)]
                >>> beam = beamtools.DuratedComplexBeamSpanner(staff[:], durations, 1)
                >>> beam.span
                1

            Set top-level beam count::

                >>> staff = Staff("c'16 d'16 e'16 f'16")
                >>> durations = [Duration(1, 8), Duration(1, 8)]
                >>> beam = beamtools.DuratedComplexBeamSpanner(staff[:], durations, 1)
                >>> beam.span = 2
                >>> beam.span
                2

            Set nonnegative integer.
            '''
            return self._span
        def fset(self, arg):
            assert isinstance(arg, (int, type(None)))
            self._span = arg
        return property(**locals())



from abjad.tools import durationtools
from abjad.tools import notetools


def fill_measures_in_expr_with_repeated_notes(expr, written_duration, iterctrl=None):
    '''.. versionadded:: 1.1

    Fill measures in `expr` with repeated notes.
    '''
    from abjad.tools import contexttools
    from abjad.tools import iterationtools

    if iterctrl is None:
        iterctrl = lambda measure, i: True
    written_duration = durationtools.Duration(written_duration)
    for i, measure in enumerate(iterationtools.iterate_measures_in_expr(expr)):
        if iterctrl(measure, i):
            time_signature = contexttools.get_effective_time_signature(measure)
            total_duration = time_signature.duration
            prolation = time_signature.implied_prolation
            notes = notetools.make_repeated_notes_with_shorter_notes_at_end(
                0, written_duration, total_duration, prolation)
            measure[:] = notes



from abjad.tools.scoretools.StaffGroup.StaffGroup import StaffGroup


class GrandStaff(StaffGroup):
    r'''Abjad model of grand staff::

        >>> staff_1 = Staff("c'4 d'4 e'4 f'4 g'1")
        >>> staff_2 = Staff("g2 f2 e1")

    ::

        >>> grand_staff = scoretools.GrandStaff([staff_1, staff_2])

    ::

        >>> f(grand_staff)
        \new GrandStaff <<
            \new Staff {
                c'4
                d'4
                e'4
                f'4
                g'1
            }
            \new Staff {
                g2
                f2
                e1
            }
        >>

    Return grand staff.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, music, context_name='GrandStaff', name=None):
        StaffGroup.__init__(self, music=music, context_name=context_name, name=name)



from abjad.tools import stafftools


def iterate_staves_in_expr(expr, reverse=False, start=0, stop=None):
    r'''.. versionadded:: 2.10

    Iterate staves forward in `expr`::

        >>> score = Score(4 * Staff([]))

    ::

        >>> f(score)
        \new Score <<
            \new Staff {
            }
            \new Staff {
            }
            \new Staff {
            }
            \new Staff {
            }
        >>

    ::

        >>> for staff in iterationtools.iterate_staves_in_expr(score):
        ...     staff
        ...
        Staff{}
        Staff{}
        Staff{}
        Staff{}

    Iterate staves backward in `expr`::

    ::

        >>> for staff in iterationtools.iterate_staves_in_expr(score, reverse=True):
        ...     staff
        ...
        Staff{}
        Staff{}
        Staff{}
        Staff{}

    Return generator.
    '''
    from abjad.tools import iterationtools

    return iterationtools.iterate_components_in_expr(
        expr, stafftools.Staff, reverse=reverse, start=start, stop=stop)



import fractions


def calculate_depth_density_of_intervals_in_interval(intervals, interval):
    '''Return a Fraction, of the duration of each interval in the
    depth tree of `intervals` within `interval`, multiplied by the depth at that interval,
    divided by the overall duration of `intervals`.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    split_tree = timeintervaltools.split_intervals_at_rationals(tree, [interval.start, interval.stop])
    split_tree = timeintervaltools.TimeIntervalTree(
        split_tree.find_intervals_starting_and_stopping_within_interval(interval))

    if not split_tree:
        return fractions.Fraction(0)

    return fractions.Fraction(sum([x.duration for x in split_tree])) / interval.duration



from fractions import Fraction
from abjad.tools import contexttools
from abjad.tools import durationtools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import sequencetools


def tempo_scaled_durations_to_q_events(durations, tempo):
    '''Convert a list of rational durations to a list of
    ``QEvent`` instances.

    Negative duration values can be used to indicate silence.  Any resulting
    pitched ``QEvents`` will default to using middle-C.

    ::

        >>> durations = [Duration(-1, 2), Duration(1, 4), Duration(1, 6)]
        >>> tempo = contexttools.TempoMark((1, 4), 55)
        >>> for x in quantizationtools.tempo_scaled_durations_to_q_events(
        ...     durations, tempo): x
        ...
        quantizationtools.SilentQEvent(
            durationtools.Offset(0, 1),
            attachments=()
            )
        quantizationtools.PitchedQEvent(
            durationtools.Offset(24000, 11),
            (NamedChromaticPitch("c'"),),
            attachments=()
            )
        quantizationtools.PitchedQEvent(
            durationtools.Offset(36000, 11),
            (NamedChromaticPitch("c'"),),
            attachments=()
            )
        quantizationtools.TerminalQEvent(
            durationtools.Offset(4000, 1)
            )

    Return a list of ``QEvent`` instances.
    '''
    from abjad.tools import quantizationtools

    durations = [durationtools.Duration(x) for x in durations]
    assert isinstance(tempo, contexttools.TempoMark)

    durations = [x for x in sequencetools.sum_consecutive_sequence_elements_by_sign(durations, sign=[-1]) if x]
    durations = [quantizationtools.tempo_scaled_duration_to_milliseconds(x, tempo) for x in durations]

    offsets = mathtools.cumulative_sums_zero([abs(x) for x in durations])

    q_events = []
    for pair in zip(offsets, durations):
        offset = durationtools.Offset(pair[0])
        duration = pair[1]
        if duration < 0: # negative duration indicates silence
            q_event = quantizationtools.SilentQEvent(offset)
        else: # otherwise, use middle-C
            q_event = quantizationtools.PitchedQEvent(offset, [0])
        q_events.append(q_event)

    # insert terminating silence QEvent
    q_events.append(quantizationtools.TerminalQEvent(offsets[-1]))

    return q_events



from abjad.tools import componenttools
from abjad.tools import spannertools


def get_nontrivial_tie_chains_masked_by_components(components):
    r'''Get nontrivial tie chains masked by `components`::

        >>> staff = Staff("c'8 ~ c'4 d'8 ~ d'4 e'4.")

    ::

        >>> f(staff)
        \new Staff {
            c'8 ~
            c'4
            d'8 ~
            d'4
            e'4.
        }

    Return only nontrivial tie chains::

        >>> tietools.get_nontrivial_tie_chains_masked_by_components(staff.leaves)
        [TieChain(Note("c'8"), Note("c'4")), TieChain(Note("d'8"), Note("d'4"))]

    Return 'masked' tie chains when only some notes of a tie chain are passed in::

        >>> tietools.get_nontrivial_tie_chains_masked_by_components(staff.leaves[1:2])
        [TieChain(Note("c'4"),)]


    Return list of zero or more (possibly masked) tie chains.

    .. versionchanged:: 2.9
        renamed ``tietools.get_tie_chains_in_expr()`` to
        ``tietools.get_nontrivial_tie_chains_masked_by_components()``.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import tietools

    assert componenttools.all_are_components(components)

    # collect tie spanners
    tie_spanners = []
    for component in components:
        spanners = spannertools.get_spanners_attached_to_component(component, tietools.TieSpanner)
        if spanners:
            spanner = spanners.pop()
            if not spanner in tie_spanners:
                tie_spanners.append(spanner)

    # initialize tie chains
    result = []
    leaves_in_components = set(iterationtools.iterate_leaves_in_expr(components))
    for spanner in tie_spanners:
        leaves_intersecting = []
        for leaf in spanner.leaves:
            if leaf in leaves_in_components:
                leaves_intersecting.append(leaf)
        result.append(tietools.TieChain(tuple(leaves_intersecting)))

    # return result
    return result



import numbers


def get_named_chromatic_pitch_from_pitch_carrier(pitch_carrier):
    '''.. versionadded:: 1.1

    Get named chromatic pitch from `pitch_carrier`::

        >>> pitch = pitchtools.NamedChromaticPitch('df', 5)
        >>> pitch
        NamedChromaticPitch("df''")
        >>> pitchtools.get_named_chromatic_pitch_from_pitch_carrier(pitch)
        NamedChromaticPitch("df''")

    ::

        >>> note = Note(('df', 5), (1, 4))
        >>> note
        Note("df''4")
        >>> pitchtools.get_named_chromatic_pitch_from_pitch_carrier(note)
        NamedChromaticPitch("df''")

    ::

        >>> note = Note(('df', 5), (1, 4))
        >>> note.note_head
        NoteHead("df''")
        >>> pitchtools.get_named_chromatic_pitch_from_pitch_carrier(note.note_head)
        NamedChromaticPitch("df''")

    ::

        >>> chord = Chord([('df', 5)], (1, 4))
        >>> chord
        Chord("<df''>4")
        >>> pitchtools.get_named_chromatic_pitch_from_pitch_carrier(chord)
        NamedChromaticPitch("df''")

    ::

        >>> pitchtools.get_named_chromatic_pitch_from_pitch_carrier(13)
        NamedChromaticPitch("cs''")

    Raise missing pitch error when `pitch_carrier` carries no pitch.

    Raise extra pitch error when `pitch_carrier` carries more than one pitch.

    Return named chromatic pitch.

    .. versionchanged:: 2.0
        renamed ``pitchtools.get_pitch()`` to
        ``pitchtools.get_named_chromatic_pitch_from_pitch_carrier()``.
    '''
    from abjad.tools import chordtools
    from abjad.tools import notetools
    from abjad.tools import pitchtools

    if isinstance(pitch_carrier, pitchtools.NamedChromaticPitch):
        return pitch_carrier
    elif isinstance(pitch_carrier, numbers.Number):
        return pitchtools.NamedChromaticPitch(pitch_carrier)
    elif isinstance(pitch_carrier, notetools.Note):
        pitch = pitch_carrier.written_pitch
        if pitch is not None:
            return get_named_chromatic_pitch_from_pitch_carrier(pitch)
        else:
            raise MissingPitchError
    elif isinstance(pitch_carrier, notetools.NoteHead):
        pitch = pitch_carrier.written_pitch
        if pitch is not None:
            return get_named_chromatic_pitch_from_pitch_carrier(pitch)
        else:
            raise MissingPitchError
    elif isinstance(pitch_carrier, chordtools.Chord):
        pitches = pitch_carrier.written_pitches
        if len(pitches) == 0:
            raise MissingPitchError
        elif len(pitches) == 1:
            return get_named_chromatic_pitch_from_pitch_carrier(pitches[0])
        else:
            raise ExtraPitchError
    else:
        raise TypeError('%s must be Pitch, Note, NoteHead or Chord.' % pitch_carrier)



from abjad.tools import spannertools


def iterate_pitched_tie_chains_in_expr(expr, reverse=False):
    r'''.. versionadded:: 2.10

    Iterate pitched tie chains forward in `expr`::

        >>> staff = Staff(r"c'4 ~ \times 2/3 { c'16 d'8 } e'8 r8 f'8 ~ f'16 r8.")

    ::

        >>> f(staff)
        \new Staff {
            c'4 ~
            \times 2/3 {
                c'16
                d'8
            }
            e'8
            r8
            f'8 ~
            f'16
            r8.
        }

    ::

        >>> for x in tietools.iterate_pitched_tie_chains_in_expr(staff):
        ...     x
        ... 
        TieChain(Note("c'4"), Note("c'16"))
        TieChain(Note("d'8"),)
        TieChain(Note("e'8"),)
        TieChain(Note("f'8"), Note("f'16"))

    Iterate pitched tie chains backward in `expr`::

    ::

        >>> for x in tietools.iterate_pitched_tie_chains_in_expr(staff, reverse=True):
        ...     x
        ... 
        TieChain(Note("f'8"), Note("f'16"))
        TieChain(Note("e'8"),)
        TieChain(Note("d'8"),)
        TieChain(Note("c'4"), Note("c'16"))

    Tie chains are pitched if they comprise notes or chords.

    Tie chains are not pitched if they comprise rests or skips.

    Return generator.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import tietools

    if not reverse:
        for leaf in iterationtools.iterate_notes_and_chords_in_expr(expr):
            tie_spanners = spannertools.get_spanners_attached_to_component(leaf, tietools.TieSpanner)
            if not tie_spanners or tuple(tie_spanners)[0]._is_my_last_leaf(leaf):
                yield tietools.get_tie_chain(leaf)
    else:
        for leaf in iterationtools.iterate_notes_and_chords_in_expr(expr, reverse=True):
            tie_spanners = spannertools.get_spanners_attached_to_component(leaf, tietools.TieSpanner)
            if not tie_spanners or tuple(tie_spanners)[0]._is_my_first_leaf(leaf):
                yield tietools.get_tie_chain(leaf)




from abjad import *


def make_nested_tuplet(tuplet_duration, outer_tuplet_proportions, inner_tuplet_subdivision_count):
    outer_tuplet = tuplettools.make_tuplet_from_duration_and_ratio(
        tuplet_duration, outer_tuplet_proportions)
    inner_tuplet_proportions = inner_tuplet_subdivision_count * [1]
    right_tie_chain = tietools.get_tie_chain(outer_tuplet.leaves[-1])
    tietools.tie_chain_to_tuplet_with_ratio(right_tie_chain, inner_tuplet_proportions)
    return outer_tuplet




# -*- encoding: utf-8 -*-
import math
from abjad import *


def make_desordre_cell(pitches):
    '''The function constructs and returns a *Désordre cell*.
    `pitches` is a list of numbers or, more generally, pitch tokens.
    '''
    notes = [Note(pitch, (1, 8)) for pitch in pitches]
    beamtools.BeamSpanner(notes)
    spannertools.SlurSpanner(notes)
    contexttools.DynamicMark('f')(notes[0])
    contexttools.DynamicMark('p')(notes[1])

    # make the lower voice
    lower_voice = Voice(notes)
    lower_voice.name = 'RH Lower Voice'
    marktools.LilyPondCommandMark('voiceTwo')(lower_voice)
    n = int(math.ceil(len(pitches) / 2.))
    chord = Chord([pitches[0], pitches[0] + 12], (n, 8))
    marktools.Articulation('>')(chord)

    # make the upper voice
    upper_voice = Voice([chord])
    upper_voice.name = 'RH Upper Voice'
    marktools.LilyPondCommandMark('voiceOne')(upper_voice)

    # combine them together
    container = Container([lower_voice, upper_voice])
    container.is_parallel = True

    # make all 1/8 beats breakable
    for leaf in lower_voice.leaves[:-1]:
        marktools.BarLine('')(leaf)

    return container




from abjad.tools import markuptools
from abjad.tools import schemetools
from abjad.tools.spannertools.TextSpanner import TextSpanner


class BracketSpanner(TextSpanner):
    r'''Abjad bracket spanner::

        >>> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        >>> spannertools.BracketSpanner(staff[:])
        BracketSpanner(c'8, d'8, e'8, f'8)

    ::

        >>> f(staff)
        \new Staff {
            \override TextSpanner #'bound-details #'left #'text = \markup { 
                \draw-line #'(0 . -1) }
            \override TextSpanner #'bound-details #'left-broken #'text = ##f
            \override TextSpanner #'bound-details #'right #'text = \markup { 
                \draw-line #'(0 . -1) }
            \override TextSpanner #'bound-details #'right-broken #'text = ##f
            \override TextSpanner #'color = #red
            \override TextSpanner #'dash-fraction = #1
            \override TextSpanner #'staff-padding = #2
            \override TextSpanner #'thickness = #1.5
            c'8 \startTextSpan
            d'8
            e'8
            f'8 \stopTextSpan
            \revert TextSpanner #'bound-details #'left #'text
            \revert TextSpanner #'bound-details #'left-broken #'text
            \revert TextSpanner #'bound-details #'right #'text
            \revert TextSpanner #'bound-details #'right-broken #'text
            \revert TextSpanner #'color
            \revert TextSpanner #'dash-fraction
            \revert TextSpanner #'staff-padding
            \revert TextSpanner #'thickness
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Render 1.5-unit thick solid red spanner.

    Draw nibs at beginning and end of spanner.

    Do not draw nibs at line breaks.

    Return bracket spanner.
    '''

    ### INITIALIZER ###

    def __init__(self, components=None):
        TextSpanner.__init__(self, components)
        markup = markuptools.Markup(markuptools.MarkupCommand('draw-line', schemetools.SchemePair(0, -1)))
        self.override.text_spanner.bound_details__left__text = markup
        self.override.text_spanner.bound_details__left_broken__text = False
        markup = markuptools.Markup(markuptools.MarkupCommand('draw-line', schemetools.SchemePair(0, -1)))
        self.override.text_spanner.bound_details__right__text = markup
        self.override.text_spanner.bound_details__right_broken__text = False
        self.override.text_spanner.color = 'red'
        self.override.text_spanner.dash_fraction = 1
        self.override.text_spanner.staff_padding = 2
        self.override.text_spanner.thickness = 1.5



def all_are_spanners(expr):
    '''.. versionadded:: 2.6

    True when `expr` is a sequence of Abjad spanners::

        >>> voice = Voice("c'8 d'8 e'8 f'8")
        >>> spanner = beamtools.BeamSpanner(voice[:2])

    ::

        >>> spannertools.all_are_spanners([spanner])
        True

    True when `expr` is an empty sequence::

        >>> spannertools.all_are_spanners([])
        True

    Otherwise false::

        >>> spannertools.all_are_spanners('foo')
        False

    Return boolean.
    '''
    from abjad.tools import spannertools

    return all([isinstance(x, spannertools.Spanner) for x in expr])



from abjad.tools import componenttools


def all_are_contexts(expr):
    '''.. versionadded:: 2.10

    True when `expr` is a sequence of Abjad contexts::

        >>> contexts = 3 * Voice("c'8 d'8 e'8")

    ::

        >>> contexttools.all_are_contexts(contexts)
        True

    True when `expr` is an empty sequence::

        >>> contexttools.all_are_contexts([])
        True

    Otherwise false::

        >>> contexttools.all_are_contexts('foo')
        False

    Return boolean.

    Function wraps ``componenttools.all_are_components()``.
    '''
    from abjad.tools import contexttools

    return componenttools.all_are_components(expr, klasses=(contexttools.Context,))



from abjad import *


def test_labeltools_label_tie_chains_in_expr_with_tie_chain_duration_01():

    staff = Staff(notetools.make_repeated_notes(4))
    tuplettools.FixedDurationTuplet(Duration(2, 8), staff[:3])
    tietools.TieSpanner(staff.leaves[:2])
    tietools.TieSpanner(staff.leaves[2:])
    labeltools.label_tie_chains_in_expr_with_tie_chain_duration(staff)

    r'''
    \new Staff {
        \times 2/3 {
            c'8 ~
                _ \markup {
                    \small
                        1/6
                    }
            c'8
            c'8 ~
                _ \markup {
                    \small
                        5/24
                    }
        c'8
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert staff.lilypond_format == "\\new Staff {\n\t\\times 2/3 {\n\t\tc'8 ~\n\t\t\t_ \\markup {\n\t\t\t\t\\small\n\t\t\t\t\t1/6\n\t\t\t\t}\n\t\tc'8\n\t\tc'8 ~\n\t\t\t_ \\markup {\n\t\t\t\t\\small\n\t\t\t\t\t5/24\n\t\t\t\t}\n\t}\n\tc'8\n}"



from abjad import *


def test_Parentage_root_01():

    tuplet = tuplettools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
    staff = Staff([tuplet])

    assert staff.parentage.root is staff
    assert tuplet.parentage.root is staff
    assert staff.leaves[0].parentage.root is staff
    assert staff.leaves[1].parentage.root is staff
    assert staff.leaves[2].parentage.root is staff



from abjad.tools.rhythmmakertools.DivisionBurnishedTaleaRhythmMaker import \
    DivisionBurnishedTaleaRhythmMaker


class TaleaRhythmMaker(DivisionBurnishedTaleaRhythmMaker):
    r'''.. versionadded:: 2.8

    Talea rhythm-maker.

    Example 1. Basic usage.

    Configure the rhythm-maker at initialization::

        >>> talea, talea_denominator, prolation_addenda = [-1, 4, -2, 3], 16, [3, 4]
        >>> maker = rhythmmakertools.TaleaRhythmMaker(
        ...     talea, talea_denominator, prolation_addenda)

    Then call the rhythm-maker on arbitrary divisions::

        >>> divisions = [(2, 8), (5, 8)]
        >>> music = maker(divisions)

    The resulting Abjad objects can be included in any score and the rhythm-maker
    can be called indefinitely on other arbitrary sequences of divisions::

        >>> music = sequencetools.flatten_sequence(music)
        >>> measures = measuretools.make_measures_with_full_measure_spacer_skips(divisions)
        >>> staff = Staff(measures)
        >>> measures = measuretools.replace_contents_of_measures_in_expr(staff, music)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                \times 4/7 {
                    r16
                    c'4
                    r8
                }
            }
            {
                \time 5/8
                \fraction \times 5/7 {
                    c'8.
                    r16
                    c'4
                    r8
                    c'8.
                    r16
                }
            }
        }

    Example 2. Tie split notes.

        >>> maker = rhythmmakertools.TaleaRhythmMaker([5], 16, tie_split_notes=True)

    Then call the rhythm-maker on arbitrary divisions::

        >>> divisions = [(2, 8), (2, 8), (2, 8), (2, 8)]
        >>> music = maker(divisions)

    The resulting Abjad objects can be included in any score and the rhythm-maker
    can be called indefinitely on other arbitrary sequences of divisions::

        >>> music = sequencetools.flatten_sequence(music)
        >>> measures = measuretools.make_measures_with_full_measure_spacer_skips(divisions)
        >>> staff = Staff(measures)
        >>> measures = measuretools.replace_contents_of_measures_in_expr(staff, music)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'4 ~
            }
            {
                c'16
                c'8. ~
            }
            {
                c'8
                c'8 ~
            }
            {
                c'8.
                c'16
            }
        }

    Usage follows the two-step instantiate-then-call pattern shown here.

    Return rhythm-maker.
    '''

    ### INITIALIZER ###

    def __init__(self, talea, talea_denominator, prolation_addenda=None, secondary_divisions=None,
        talea_helper=None, prolation_addenda_helper=None, secondary_divisions_helper=None,
        beam_each_cell=False, beam_cells_together=False, tie_split_notes=False):
        lefts, middles, rights = [0], [0], [0]
        left_lengths, right_lengths = [0], [0]
        DivisionBurnishedTaleaRhythmMaker.__init__(self, talea, talea_denominator, prolation_addenda,
            lefts, middles, rights, left_lengths, right_lengths, secondary_divisions,
            talea_helper=talea_helper, prolation_addenda_helper=prolation_addenda_helper,
            secondary_divisions_helper=secondary_divisions_helper,
            beam_each_cell=beam_each_cell, beam_cells_together=beam_cells_together,
            tie_split_notes=tie_split_notes)



from abjad.tools import durationtools


def copy_governed_component_subtree_from_offset_to(component, start=0, stop=None):
    r'''.. versionadded:: 1.1

    Copy governed `component` subtree from `start` prolated duration
    to `stop` prolated duration.

    Governed subtree refers to `component` together with the 
    children of `component`::

        >>> voice = Voice(r"c'8 d'8 \times 2/3 { e'8 f'8 g'8 }")

    ::

        >>> f(voice)
        \new Voice {
           c'8
           d'8
           \times 2/3 {
              e'8
              f'8
              g'8
           }
        }

    ::

        >>> new = componenttools.copy_governed_component_subtree_from_offset_to(
        ...     voice, (0, 8), (3, 8))

    ::

        >>> f(new)
        \new Voice {
           c'8
           d'8
           \times 2/3 {
              e'8
              f'16
           }
        }

    Raise contiguity error if asked to slice a parallel container. ::

        >>> staff = Staff(Voice("c'8 d'8") * 2)
        >>> staff.is_parallel = True
        >>> f(staff)
        \new Staff <<
        \new Voice {
            c'8
            d'8
        }
        \new Voice {
            c'8
            d'8
        }
        >>

    Raise contiguity error when attempting to copy fleaves from parallel container.

    But note that cases with ``0 = start`` work correctly::

        >>> new = componenttools.copy_governed_component_subtree_from_offset_to(
        ...     voice, (0, 8), (1, 8))

    ::

        >>> f(new)
        \new Voice {
            c'8
        }

    Cases with ``0 < start`` do not work correctly::

        >>> new = componenttools.copy_governed_component_subtree_from_offset_to(
        ...     voice, (1, 8), (2, 8))

    ::

        >>> f(new)
        \new Voice {
            c'8
            d'8
        }

    Create ad hoc tuplets as required::

        >>> voice = Voice([Note("c'4")])
        >>> new = componenttools.copy_governed_component_subtree_from_offset_to(
        ...     voice, 0, (1, 12))

    ::

        >>> f(new)
        \new Voice {
            \times 2/3 {
                c'8
            }
        }

    Function does NOT copy parentage of `component` when `component` is a leaf::

        >>> voice = Voice([Note("c'4")])
        >>> new_leaf = componenttools.copy_governed_component_subtree_from_offset_to(
        ...     voice[0], 0, (1, 8))

    ::

        >>> f(new_leaf)
        c'8

    ::

        >>> new_leaf.parent is None
        True

    Return ``(untrimmed_copy, first_dif, second_dif)``.
    '''
    from abjad.tools import componenttools
    from abjad.tools import containertools
    from abjad.tools import leaftools

    # check input
    assert isinstance(component, componenttools.Component)
    start = durationtools.Duration(start)
    if start < 0:
        start = durationtools.Duration(0)
    if stop is None:
        stop = component.prolated_duration
    else:
        stop = durationtools.Duration(stop)
    assert start <= stop

    # copy component from start offset to stop offset and return
    if isinstance(component, leaftools.Leaf):
        return _copy_leaf_from_start_offset_to_stop_offset(component, start, stop)
    elif isinstance(component, containertools.Container):
        return _copy_container_from_start_offset_to_stop_offset(component, start, stop)
    else:
        raise ValueError('must be leaf or container: {!r}'.format(component))


def _copy_leaf_from_start_offset_to_stop_offset(leaf, start, stop):
    from abjad.tools import componenttools
    from abjad.tools import leaftools

    if leaf.prolated_duration <= start:
        return None

    if leaf.prolated_duration < stop:
        stop = leaf.prolated_duration

    total = stop - start
    if total == 0:
        return None

    new_leaf = componenttools.copy_components_and_fracture_crossing_spanners([leaf])[0]
    leaftools.set_preprolated_leaf_duration(new_leaf, total)

    return new_leaf


def _copy_container_from_start_offset_to_stop_offset(container, start, stop):
    from abjad.tools import componenttools
    from abjad.tools import leaftools

    # copy container
    container, first_dif, second_dif = _get_lcopy(container, start, stop)

    # get container start and stop leaves
    leaf_start = container.leaves[0]
    leaf_end = container.leaves[-1]

    # split first leaf
    leaf_start_splitted = componenttools.split_component_at_offset(
        leaf_start, first_dif, fracture_spanners=False)
    assert len(leaf_start_splitted) == 2
    if not leaf_start_splitted[0] == []:
        leaftools.remove_leaf_and_shrink_durated_parent_containers(leaf_start_splitted[0][0])

    # split second leaf
    leaf_end_splitted = componenttools.split_component_at_offset(
        leaf_end, second_dif, fracture_spanners=False)
    assert len(leaf_end_splitted) == 2
    if not leaf_end_splitted[0] == []:
        leaftools.remove_leaf_and_shrink_durated_parent_containers(
            leaf_end_splitted[1][0])

    # return adjusted container
    return container


# TODO: rename explicitly
def _get_lcopy(container, start, stop):
    from abjad.tools import componenttools
    from abjad.tools import iterationtools

    # initialize loop variables
    total_dur = durationtools.Duration(0)
    start_leaf, stop_leaf = None, None
    first_dif = second_dif = 0

    for i, leaf in enumerate(iterationtools.iterate_leaves_in_expr(container)):
        total_dur += leaf.prolated_duration
        if total_dur == start and start_leaf is None:
            start_leaf = i
            first_dif = 0
        elif start < total_dur and start_leaf is None:
            start_leaf = i
            first_dif = leaf.prolated_duration - (total_dur - start)
            #print first_dif
        if stop <= total_dur and stop_leaf is None:
            stop_leaf = i + 1
            #second_dif = leaf.prolated_duration - (total_dur - stop)
            flamingo = total_dur - stop
            if flamingo != 0:
                second_dif = leaf.prolated_duration - flamingo
            #print second_dif
            #print 'breaking after stop'
            break

    #print start_leaf, stop_leaf
    untrimmed_copy = componenttools.copy_governed_component_subtree_by_leaf_range(
        container, start_leaf, stop_leaf)

    return untrimmed_copy, first_dif, second_dif



from abjad import *


def test_TempoMark_storage_format_01():

    tempo_mark = contexttools.TempoMark('Allegro', (1, 4), 84)

    r'''
    contexttools.TempoMark(
        'Allegro',
        durationtools.Duration(1, 4),
        84
        )
    '''

    assert tempo_mark.storage_format == "contexttools.TempoMark(\n\t'Allegro',\n\tdurationtools.Duration(1, 4),\n\t84\n\t)"



import copy
from abjad.tools import chordtools
from abjad.tools import marktools
from abjad.tools import notetools
from abjad.tools import tietools


def edit_cello_voice(score, durated_reservoir):

    voice = score['Cello Voice']
    descents = durated_reservoir['Cello']

    tie_chain = tietools.get_tie_chain(voice[-1])
    for leaf in tie_chain.leaves:
        parent = leaf.parent
        index = parent.index(leaf)
        parent[index] = chordtools.Chord(['e,', 'a,'], leaf.written_duration)

    unison_descent = copy.deepcopy(voice[-len(descents[-1]):])
    voice.extend(unison_descent)
    for chord in unison_descent:
        index = chord.parent.index(chord)
        parent[index] = notetools.Note(chord.written_pitches[1], chord.written_duration)
        marktools.Articulation('accent')(parent[index])
        marktools.Articulation('tenuto')(parent[index])

    voice.extend('a,1. ~ a,2 b,1 ~ b,1. ~ b,1. a,1. ~ a,1. ~ a,1. ~ a,1. ~ a,1. ~ a,2 r4 r2.')




from abjad.tools.developerscripttools.DirectoryScript import DirectoryScript
from abjad.tools.developerscripttools.ReplaceInFilesScript import ReplaceInFilesScript
import os


class ReplacePromptsScript(DirectoryScript):
    '''Replace prompts in code examples recursively:

    ::

        bash$ ajv replace prompts -h
        usage: replace-prompts [-h] [--version] [-PA | -AP] path

        Replace prompts.

        positional arguments:
          path                  directory tree to be recursed over

        optional arguments:
          -h, --help            show this help message and exit
          --version             show program's version number and exit
          -PA, --python-to-abjad
                                replace Python (">>>") prompts with Abjad ("abjad>")
                                prompts
          -AP, --abjad-to-python
                                replace Abjad ("abjad>") prompts with Python (">>>")
                                prompts

        examples:

          Replace Python prompts with Abjad prompts in the current directory:

          $ abj-dev replace prompts --python-to-abjad .

          Replace Abjad prompts with Python prompts in the grandparent directory:

          $ abj-dev replace prompts --abjad-to-python ../..
            
    `ReplacePromptsScript` uses `ReplaceInFilesScript` for its replacement functionality.

    Return `ReplacePromptsScript` instance.
    '''

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def alias(self):
        return 'prompts'

    @property
    def long_description(self):
        return '''\
examples:

  Replace Python prompts with Abjad prompts in the current directory:

  $ abj-dev replace prompts --python-to-abjad .

  Replace Abjad prompts with Python prompts in the grandparent directory:

  $ abj-dev replace prompts --abjad-to-python ../..
    '''

    @property
    def scripting_group(self):
        return 'replace'

    @property
    def short_description(self):
        return 'Replace prompts.'

    @property
    def version(self):
        return 1.0

    ### PUBLIC METHODS ###

    def process_args(self, args):
        if args.mode is None:
            print 'No replacement pattern specified, exiting.'

        script = ReplaceInFilesScript()
        arguments = [args.path] + args.mode + ['--force']
        script(arguments)

    def setup_argument_parser(self, parser):

        parser.add_argument('path',
            type=self._validate_path,
            help='directory tree to be recursed over'
            )

        group = parser.add_mutually_exclusive_group()

        group.add_argument('-PA', '--python-to-abjad',
            action='store_const',
            const=['>>> ', 'abjad> '],
            dest='mode',
            help='replace Python (">>>") prompts with Abjad ("abjad>") prompts'
            )

        group.add_argument('-AP', '--abjad-to-python',
            action='store_const',
            const=['abjad> ', '>>> '],
            dest='mode',
            help='replace Abjad ("abjad>") prompts with Python (">>>") prompts'
            )

        parser.set_defaults(mode=None)



from abjad import *


def test_componenttools_partition_components_by_durations_exactly_01():

    t = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 4)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)
    contexttools.TempoMark(Duration(1, 4), 60, target_context=Staff)(t)

    r'''
    \new Staff {
            \time 2/8
            \tempo 4=60
            c'8
            d'8
            \time 2/8
            e'8
            f'8
            \time 2/8
            g'8
            a'8
            \time 2/8
            b'8
            c''8
            %% tempo 4=60 ends here
    }
    '''

    parts = componenttools.partition_components_by_durations_exactly(
        t.leaves, [1.5], cyclic=True, in_seconds=True, overhang=False)

    "[[Note(c'', 8), Note(b', 8), Note(a', 8)], [Note(g', 8), Note(f', 8), Note(e', 8)]]"

    assert len(parts) == 2
    assert parts[0] == list(t.leaves[:3])
    assert parts[1] == list(t.leaves[3:6])


def test_componenttools_partition_components_by_durations_exactly_02():

    t = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 4)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)
    contexttools.TempoMark(Duration(1, 4), 60, target_context = Staff)(t)

    r'''
    \new Staff {
            \time 2/8
            \tempo 4=60
            c'8
            d'8
            \time 2/8
            e'8
            f'8
            \time 2/8
            g'8
            a'8
            \time 2/8
            b'8
            c''8
            %% tempo 4=60 ends here
    }
    '''

    parts = componenttools.partition_components_by_durations_exactly(
        t.leaves, [1.5], cyclic=True, in_seconds=True, overhang=True)

    "[[Note(c'', 8), Note(b', 8), Note(a', 8)], "
    "[Note(g', 8), Note(f', 8), Note(e', 8)], "
    "[Note(d', 8), Note(c', 8)]]"

    assert len(parts) == 3
    assert parts[0] == list(t.leaves[:3])
    assert parts[1] == list(t.leaves[3:6])
    assert parts[2] == list(t.leaves[6:8])


def test_componenttools_partition_components_by_durations_exactly_03():

    t = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 4)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)

    r'''
    \new Staff {
            \time 2/8
            c'8
            d'8
            \time 2/8
            e'8
            f'8
            \time 2/8
            g'8
            a'8
            \time 2/8
            b'8
            c''8
    }
    '''

    parts = componenttools.partition_components_by_durations_exactly(
        t.leaves, [Duration(3, 8)], cyclic=True, in_seconds=False, overhang=True)

    "[[Note(c'', 8), Note(b', 8), Note(a', 8)], "
    "[Note(g', 8), Note(f', 8), Note(e', 8)], "
    "[Note(d', 8), Note(c', 8)]]"

    assert len(parts) == 3
    assert parts[0] == list(t.leaves[:3])
    assert parts[1] == list(t.leaves[3:6])
    assert parts[2] == list(t.leaves[6:8])


def test_componenttools_partition_components_by_durations_exactly_04():

    t = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 4)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)
    contexttools.TempoMark(Duration(1, 4), 60, target_context = Staff)(t)

    r'''
    \new Staff {
            \time 2/8
            \tempo 4=60
            c'8
            d'8
            \time 2/8
            e'8
            f'8
            \time 2/8
            g'8
            a'8
            \time 2/8
            b'8
            c''8
            %% tempo 4=60 ends here
    }
    '''

    parts = componenttools.partition_components_by_durations_exactly(
        t.leaves, [1.5], cyclic=False, in_seconds=True, overhang=False)

    "[[Note(c'', 8), Note(b', 8), Note(a', 8)]]"

    assert len(parts) == 1
    assert parts[0] == list(t.leaves[:3])


def test_componenttools_partition_components_by_durations_exactly_05():

    t = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 4)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)

    r'''
    \new Staff {
            \time 2/8
            c'8
            d'8
            \time 2/8
            e'8
            f'8
            \time 2/8
            g'8
            a'8
            \time 2/8
            b'8
            c''8
    }
    '''

    groups = \
        componenttools.partition_components_by_durations_exactly(
        t.leaves, [Duration(3, 8)], cyclic=False, in_seconds=False, overhang=False)

    "[[Note(c'', 8), Note(b', 8), Note(a', 8)]]"

    assert len(groups) == 1
    assert groups[0] == list(t.leaves[:3])



from abjad import *


def test_componenttools_sum_duration_of_components_01():
    '''Sum of prolated durations of components in list.
    '''

    t = tuplettools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")

    assert componenttools.sum_duration_of_components(t.leaves) == Duration(2, 8)


def test_componenttools_sum_duration_of_components_02():
    '''Return sum of preprolated duration of components in list.
    '''

    t = tuplettools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")

    assert componenttools.sum_duration_of_components(t[:], preprolated=True) == Duration(3, 8)


def test_componenttools_sum_duration_of_components_03():
    '''Return zero for empty list.
    '''

    assert componenttools.sum_duration_of_components([], preprolated=True) == Duration(0)



from abjad.tools import stringtools
from abjad.tools.spannertools.DirectedSpanner.DirectedSpanner import DirectedSpanner


class SlurSpanner(DirectedSpanner):
    r'''Abjad slur spanner::

        >>> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        >>> spannertools.SlurSpanner(staff[:])
        SlurSpanner(c'8, d'8, e'8, f'8)

    ::

        >>> f(staff)
        \new Staff {
            c'8 (
            d'8
            e'8
            f'8 )
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Return slur spanner.
    '''

    ### INITIALIZER ###

    def __init__(self, components=None, direction=None):
        DirectedSpanner.__init__(self, components, direction)

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        DirectedSpanner._copy_keyword_args(self, new)

    def _format_right_of_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            direction = self.direction
            if direction is not None:
                result.append('{} ('.format(stringtools.arg_to_tridirectional_lilypond_symbol(direction)))
            else:
                result.append('(')
        if self._is_my_last_leaf(leaf):
            result.append(')')
        return result



from abjad.tools import containertools
from abjad.tools import durationtools
from abjad.tools import mathtools


def move_measure_prolation_to_full_measure_tuplet(expr):
    '''.. versionadded:: 2.0

    Move measure prolation to full-measure tuplet.

    Turn non-power-of-two measures into power-of-two measures containing 
    a single fixed-duration tuplet.

    Note that not all non-power-of-two measures can be made power-of-two.

    Returns None because processes potentially many measures.

    .. versionchanged:: 2.0
        renamed ``measuretools.project()`` to
        ``measuretools.move_measure_prolation_to_full_measure_tuplet()``.
    '''
    from abjad.tools import contexttools
    from abjad.tools import iterationtools
    from abjad.tools import measuretools
    from abjad.tools import timesignaturetools
    from abjad.tools import tuplettools

    for measure in iterationtools.iterate_measures_in_expr(expr):
        if contexttools.get_effective_time_signature(measure).has_non_power_of_two_denominator:

            # find time signature and contents multipliers
            time_signature_multiplier = contexttools.get_effective_time_signature(measure).implied_prolation
            contents_multiplier = measuretools.get_likely_multiplier_of_components(measure[:])

            # update non-power-of-two time signature to power-of-two
            effective_time_signature = contexttools.get_effective_time_signature(measure)
            power_of_two_time_signature = effective_time_signature.with_power_of_two_denominator(
                contents_multiplier)
            contexttools.detach_time_signature_marks_attached_to_component(measure)
            power_of_two_time_signature.attach(measure)

            # find target duration and create tuplet
            target_duration = time_signature_multiplier * measure.contents_duration
            tuplet = tuplettools.FixedDurationTuplet(target_duration, measure[:])

            # scale tuplet contents, if helpful
            if contents_multiplier is not None:
                #containertools.scale_contents_of_container(tuplet, ~contents_multiplier)
                inverse_multiplier = durationtools.Duration(
                    contents_multiplier.denominator, contents_multiplier.numerator)
                containertools.scale_contents_of_container(tuplet, inverse_multiplier)



from abjad.tools import contexttools
from abjad.tools import pitchtools
from abjad.tools import voicetools


def iterate_notes_and_chords_in_expr_outside_traditional_instrument_ranges(expr):
    '''.. versionadded:: 2.0

    Iterate notes and chords in `expr` outside traditional instrument ranges::

        >>> staff = Staff("c'8 r8 <d fs>8 r8")
        >>> instrumenttools.Violin()(staff)
        Violin()(Staff{4})

    ::

        >>> list(
        ... instrumenttools.iterate_notes_and_chords_in_expr_outside_traditional_instrument_ranges(
        ... staff))
        [Chord('<d fs>8')]

    Return generator.
    '''
    from abjad.tools import iterationtools

    for note_or_chord in iterationtools.iterate_notes_and_chords_in_expr(expr):
        instrument = contexttools.get_effective_instrument(note_or_chord)
        if instrument is None:
            raise MissingInstrumentError
        if note_or_chord not in instrument.traditional_pitch_range:
            yield note_or_chord



from abjad.tools.timeintervaltools import *


def test_TimeIntervalTreeDictionary_storage_format_01():

    a = TimeIntervalTree([TimeInterval(0, 1, {'name': 'a'})])
    b = TimeIntervalTree([TimeInterval(1, 2, {'name': 'b'})])
    c = TimeIntervalTree([TimeInterval(0, 3, {'name': 'c'})])
    d = TimeIntervalTree([TimeInterval(2, 3, {'name': 'd'})])

    treedict = TimeIntervalTreeDictionary({'a': a, 'b': b, 'c': c, 'd': d})
    
    r'''
    timeintervaltools.TimeIntervalTreeDictionary({
        'a': timeintervaltools.TimeIntervalTree([
            timeintervaltools.TimeInterval(
                Offset(0, 1),
                Offset(1, 1),
                {
                    'name': 'a',
                }),
            ]),
        'c': timeintervaltools.TimeIntervalTree([
            timeintervaltools.TimeInterval(
                Offset(0, 1),
                Offset(3, 1),
                {
                    'name': 'c',
                }),
            ]),
        'b': timeintervaltools.TimeIntervalTree([
            timeintervaltools.TimeInterval(
                Offset(1, 1),
                Offset(2, 1),
                {
                    'name': 'b',
                }),
            ]),
        'd': timeintervaltools.TimeIntervalTree([
            timeintervaltools.TimeInterval(
                Offset(2, 1),
                Offset(3, 1),
                {
                    'name': 'd',
                }),
            ]),
        })
    '''

    assert treedict.storage_format == "timeintervaltools.TimeIntervalTreeDictionary({\n\t'a': timeintervaltools.TimeIntervalTree([\n\t\ttimeintervaltools.TimeInterval(\n\t\t\tOffset(0, 1),\n\t\t\tOffset(1, 1),\n\t\t\t{\n\t\t\t\t'name': 'a',\n\t\t\t}),\n\t\t]),\n\t'c': timeintervaltools.TimeIntervalTree([\n\t\ttimeintervaltools.TimeInterval(\n\t\t\tOffset(0, 1),\n\t\t\tOffset(3, 1),\n\t\t\t{\n\t\t\t\t'name': 'c',\n\t\t\t}),\n\t\t]),\n\t'b': timeintervaltools.TimeIntervalTree([\n\t\ttimeintervaltools.TimeInterval(\n\t\t\tOffset(1, 1),\n\t\t\tOffset(2, 1),\n\t\t\t{\n\t\t\t\t'name': 'b',\n\t\t\t}),\n\t\t]),\n\t'd': timeintervaltools.TimeIntervalTree([\n\t\ttimeintervaltools.TimeInterval(\n\t\t\tOffset(2, 1),\n\t\t\tOffset(3, 1),\n\t\t\t{\n\t\t\t\t'name': 'd',\n\t\t\t}),\n\t\t]),\n\t})"



from abjad.tools.sequencetools.is_integer_equivalent_pair import is_integer_equivalent_pair


def is_fraction_equivalent_pair(expr):
    r'''.. versionadded:: 2.9

    True when `expr` is an integer-equivalent pair of numbers excluding ``0`` as the second term::

        >>> sequencetools.is_fraction_equivalent_pair((2, 3))
        True

    Otherwise false::

        >>> sequencetools.is_fraction_equivalent_pair((2, 0))
        False

    Return boolean.
    '''

    return is_integer_equivalent_pair(expr) and not expr[1] == 0



from abjad.tools import contexttools
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools.instrumenttools._Clarinet._Clarinet import _Clarinet


class BFlatClarinet(_Clarinet):
    r'''.. versionadded:: 2.0

    Abjad model of the B-flat clarinet::

        >>> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        >>> instrumenttools.BFlatClarinet()(staff)
        BFlatClarinet()(Staff{4})

    ::

        >>> f(staff)
        \new Staff {
            \set Staff.instrumentName = \markup { Clarinet in B-flat }
            \set Staff.shortInstrumentName = \markup { Cl. in B-flat }
            c'8
            d'8
            e'8
            f'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    The B-flat clarinet targets staff context by default.
    '''

    def __init__(self, **kwargs):
        _Clarinet.__init__(self, **kwargs)
        self._default_instrument_name = 'clarinet in B-flat'
        self._default_performer_names.extend(['clarinettist', 'clarinetist'])
        self._default_short_instrument_name = 'cl. in B-flat'
        self._is_primary_instrument = True
        self.sounding_pitch_of_fingered_middle_c = pitchtools.NamedChromaticPitch('bf')
        self.primary_clefs = [contexttools.ClefMark('treble')]
        self._copy_primary_clefs_to_all_clefs()
        self._traditional_pitch_range = pitchtools.PitchRange(-10, 34)



import abc
from abjad.tools.abctools import AbjadObject


class GraceHandler(AbjadObject):
    '''Abstract base class from which concrete ``GraceHandler`` subclasses inherit.

    Determines what pitch, if any, will be selected from a list of ``QEvents`` to
    be applied to an attack-point generated by a ``QGrid``, and whether there should be
    a ``GraceContainer`` attached to that attack-point.

    When called on a sequence of ``QEvents``, ``GraceHandler`` subclasses should return a pair,
    where the first item of the pair is a sequence of pitch tokens or ``None``, and where the
    second item of the pair is a ``GraceContainer`` instance or None.
    '''

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    @abc.abstractmethod
    def __call__(self, q_events):
        raise NotImplemented



from abjad.tools import leaftools


def make_tied_chord(pitches, duration, decrease_durations_monotonically=True):
    '''Returns a list of chords to fill the given duration.

    Chords returned are tie spanned.
    '''
    from abjad.tools import chordtools

    return leaftools.make_tied_leaf(chordtools.Chord, duration, decrease_durations_monotonically=decrease_durations_monotonically, pitches=pitches)



from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_LilyPondParser__functions__grace_01():
    target = Container([
        Note("c'4"),
        Note("d'4"),
        Note("e'2")
    ])
    grace = gracetools.GraceContainer([
        Note("g''16"),
        Note("fs''16")
    ])
    grace(target[2])

    r'''{
        c'4
        d'4
        \grace {
            g''16
            fs''16
        }
        e'2
    }
    '''

    input = r"{ c'4 d'4 \grace { g''16 fs''16} e'2 }"
    parser = LilyPondParser()
    result = parser(input)
    assert target.lilypond_format == result.lilypond_format and target is not result




from abjad.tools import *
import py


def test_Ratio___init___01():
    '''Init from integers.
    '''

    ratio = mathtools.Ratio(1, 2, 1)
    assert isinstance(ratio, mathtools.Ratio)


def test_Ratio___init___02():
    '''Init from a tuple or list.
    '''

    ratio = mathtools.Ratio((1, 2, 1))
    assert isinstance(ratio, mathtools.Ratio)


def test_Ratio___init___03():

    assert py.test.raises(Exception, 'mathtools.Ratio(1, 2, 0)')    



from abjad.tools import durationtools


def make_leaves_from_talea(talea, denominator, 
    decrease_durations_monotonically=True, tie_rests=False, use_skips=False):
    r'''.. versionadded:: 2.0

    Make leaves from `talea` and `denominator`::

        >>> leaves = leaftools.make_leaves_from_talea([3, -3, 5, -5], 8)
        >>> staff = Staff(leaves)

    ::

        >>> f(staff)
        \new Staff {
            c'4.
            r4.
            c'2 ~
            c'8
            r2
            r8
        }

    Interpret positive elements in `talea` as notes.

    Interpret negative elements in `talea` as rests.

    Set the pitch of all notes to middle C.

    When ``use_skips=False`` use skips instead of rests.

    .. note:: add skip example.

    Return list of notes and / or rests.
    '''
    from abjad.tools import leaftools

    # make notes and rests
    result = []
    for note_value in talea:
        if note_value == 0:
            raise ValueError('note values must be nonzero.')
        elif 0 < note_value:
            leaves = leaftools.make_leaves(
                [0], [durationtools.Duration(note_value, denominator)], 
                decrease_durations_monotonically=decrease_durations_monotonically, 
                tie_rests=tie_rests)
        else:
            leaves = leaftools.make_leaves(
                [None], [durationtools.Duration(-note_value, denominator)],
                decrease_durations_monotonically=decrease_durations_monotonically, 
                tie_rests=tie_rests)
        result.extend(leaves)

    # return result
    return result



def calculate_density_of_attacks_in_interval(intervals, interval):
    '''Calculate the number of attacks in `interval` over the duration of `interval`.

    Return Fraction.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    return len(tree.find_intervals_starting_within_interval(interval)) \
        / interval.duration



from abjad.tools.quantizationtools.GraceHandler import GraceHandler


class CollapsingGraceHandler(GraceHandler):
    '''A GraceHandler which collapses pitch information into a single Chord,
    rather than creating a GraceContainer.

    Return ``CollapsingGraceHandler`` instance.
    '''

    ### SPECIAL METHODS ###

    def __call__(self, q_events):
        from abjad.tools import quantizationtools
        pitches = []
        for q_event in q_events:
            if isinstance(q_event, quantizationtools.PitchedQEvent):
                pitches.extend(q_event.pitches)
        return tuple(pitches), None



from abjad import *
from abjad.demos.ferneyhough.make_rows_of_nested_tuplets import make_rows_of_nested_tuplets
from abjad.demos.ferneyhough.configure_score import configure_score
from abjad.demos.ferneyhough.configure_lilypond_file import configure_lilypond_file


def make_lilypond_file(tuplet_duration, row_count, column_count):
    rows_of_nested_tuplets = make_rows_of_nested_tuplets(tuplet_duration, row_count, column_count)
    all_nested_tuplets = sequencetools.flatten_sequence(rows_of_nested_tuplets)
    staff = stafftools.RhythmicStaff(all_nested_tuplets)
    time_signature  = contexttools.TimeSignatureMark((1, 4))
    time_signature.attach(staff)
    score = Score([staff])
    configure_score(score)
    lilypond_file = lilypondfiletools.make_basic_lilypond_file(score)
    configure_lilypond_file(lilypond_file)
    return lilypond_file




from abjad.tools.abjadbooktools.OutputFormat import OutputFormat


class LaTeXOutputFormat(OutputFormat):

    ### INITIALIZER ###

    def __init__(self):
        code_block_opening = '\\begin{lstlisting}'
        code_block_opening += '[basicstyle=\\footnotesize\\ttfamily, tabsize=4, '
        code_block_opening += 'showtabs=false, showspaces=false]\n'
        code_block_closing = '\\end{lstlisting}\n'
        code_indent = 0
        image_block = '\\includegraphics{{images/{}.pdf}}\n'
        image_format = 'pdf'
        OutputFormat.__init__(self, code_block_opening, code_block_closing,
            code_indent, image_block, image_format)



from abjad.tools.developerscripttools.DeveloperScript import DeveloperScript
import os


class MakeNewFunctionTemplateScript(DeveloperScript):
    '''Create function stub files:

    ::

        bash$ ajv new function -h
        usage: make-new-function-template [-h] [--version] (-X | -M) name

        Make a new function template file.

        positional arguments:
          name                tools package qualified function name

        optional arguments:
          -h, --help          show this help message and exit
          --version           show program's version number and exit
          -X, --experimental  use the Abjad experimental path
          -M, --mainline      use the Abjad mainline tools path

    Return `MakeNewFunctionTemplateScript` instance.
    '''

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def alias(self):
        return 'function'

    @property
    def long_description(self):
        return None

    @property
    def scripting_group(self):
        return 'new'

    @property
    def short_description(self):
        return 'Make a new function template file.'

    @property
    def version(self):
        return 1.0

    ### PRIVATE METHODS ###

    def _get_function_names_in_tools_package(self, root, tools_package_name):
        path = os.path.join(root, tools_package_name)
        return tuple(sorted([x[:-3] for x in os.listdir(path)
            if x.endswith('.py') and not x.startswith('__')]))

    def _get_function_text(self, function_name):
        return [
            'def {}():'.format(function_name),
            '    pass'
        ]

    def _get_tools_package_names(self, root):
        names = []
        for x in os.listdir(root):
            if os.path.isdir(os.path.join(root, x)):
                if not x.startswith(('_', '.')):
                    names.append(x)
        return tuple(sorted(names))

    ### PUBLIC METHODS ###

    def process_args(self, args):

        if args.name.count('.') != 1:
            print 'Error: {!r} not in tools_package.function format.'.format(args.name)
            return

        root = args.path
        tools_package_name, function_name = args.name.split('.')

        if tools_package_name not in self._get_tools_package_names(root):
            print 'Error: {!r} is not a valid tools package.'.format(tools_package_name)
            return

        if function_name in self._get_function_names_in_tools_package(root, tools_package_name):
            print 'Error: {!r} already exists in {!r}'.format(function_name, tools_package_name)
            return

        package_path = os.path.join(root, tools_package_name)

        module_path = os.path.join(package_path, '{}.py'.format(function_name))
        module_text = '\n'.join(self._get_function_text(function_name)) + '\n'

        with open(module_path, 'w') as f:
            f.write(module_text)

    def setup_argument_parser(self, parser):

        from abjad import ABJCFG

        parser.add_argument('name',
            help='tools package qualified function name'
            )

        path_group = parser.add_mutually_exclusive_group(required=True)

        path_group.add_argument('-X', '--experimental',
            action='store_const',
            const=os.path.join(ABJCFG.ABJAD_EXPERIMENTAL_PATH, 'tools'),
            dest='path',
            help='use the Abjad experimental tools path',
            )

        path_group.add_argument('-M', '--mainline',
            action='store_const',
            const=os.path.join(ABJCFG.ABJAD_PATH, 'tools'),
            dest='path',
            help='use the Abjad mainline tools path',
            )



from abjad.tools import componenttools


def iterate_components_in_expr(expr, klass=None, reverse=False, start=0, stop=None):
    r'''.. versionadded:: 2.10

    Iterate components forward in `expr`.
    '''

    klass = klass or componenttools.Component

    def component_iterator(expr, klass, reverse=False):
        if isinstance(expr, klass):
            yield expr
        if isinstance(expr, (list, tuple)) or hasattr(expr, '_music'):
            if hasattr(expr, '_music'):
                expr = expr._music 
            if reverse:
                expr = reversed(expr)
            for m in expr:
                for x in component_iterator(m, klass, reverse=reverse):
                    yield x
            
    def subrange(iter, start=0, stop=None):
        # if start<0, then 'stop-start' gives a funny result
        # dont have to check stop>=start, as xrange(stop-start) already handles that
        assert 0 <= start

        try:
            # Skip the first few elements, up to 'start' of them:
            for i in xrange(start):
                iter.next()  # no 'yield' to swallow the results

            # Now generate (stop-start) elements (or all elements if stop is None)
            if stop is None:
                for x in iter:
                    yield x
            else:
                for i in xrange(stop-start):
                    yield iter.next()
        except StopIteration:
            # This happens if we exhaust the list before
            # we generate a total of 'stop' elements
            pass

    return subrange(component_iterator(expr, klass, reverse=reverse), start, stop)



from abjad import *
import py.test


def test_Container___getitem___01():
    '''Get one container component with positive index.'''

    notes = [Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8")]
    t = Voice(notes)

    assert t[0] is notes[0]
    assert t[1] is notes[1]
    assert t[2] is notes[2]
    assert t[3] is notes[3]


def test_Container___getitem___02():
    '''Get one container component with negative index.'''

    notes = [Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8")]
    t = Voice(notes)

    assert t[-1] is notes[3]
    assert t[-2] is notes[2]
    assert t[-3] is notes[1]
    assert t[-4] is notes[0]


def test_Container___getitem___03():
    '''Get slice from container.'''

    notes = [Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8")]
    t = Voice(notes)

    assert t[:1] == notes[:1]
    assert t[:2] == notes[:2]
    assert t[:3] == notes[:3]
    assert t[:4] == notes[:4]


def test_Container___getitem___04():
    '''Bad index raises IndexError.'''

    t = Voice("c'8 d'8 e'8 f'8")

    assert py.test.raises(IndexError, 't[99]')


def test_Container___getitem___05():
    '''Get one named component in subtree rooted at container.'''

    template = scoretemplatetools.StringQuartetScoreTemplate()
    score = template()

    assert score['First Violin Staff'].name == 'First Violin Staff'
    assert score['First Violin Voice'].name == 'First Violin Voice'


def test_Container___getitem___06():
    '''Bad name raises MissingNamedComponentError.'''

    template = scoretemplatetools.StringQuartetScoreTemplate()
    score = template()

    assert py.test.raises(MissingNamedComponentError, "score['Foo']")


def test_Container___getitem___07():
    '''Duplicate named contexts raise ExtraNamedComponentError.'''

    template = scoretemplatetools.StringQuartetScoreTemplate()
    score = template()

    assert score['First Violin Voice'].name == 'First Violin Voice'

    score['Cello Staff'].append(Voice(name='First Violin Voice'))

    assert py.test.raises(ExtraNamedComponentError, "score['First Violin Voice']")

    extra_first_violin_voice = score['Cello Staff'].pop()

    assert score['First Violin Voice'].name == 'First Violin Voice'
    assert score['First Violin Voice'] is not extra_first_violin_voice



def get_clef_marks_attached_to_component(component):
    r'''.. versionadded:: 2.3

    Get clef marks attached to `component`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> contexttools.ClefMark('treble')(staff)
        ClefMark('treble')(Staff{4})

    ::

        >>> f(staff)
        \new Staff {
            \clef "treble"
            c'8
            d'8
            e'8
            f'8
        }

    ::

        >>> contexttools.get_clef_marks_attached_to_component(staff)
        (ClefMark('treble')(Staff{4}),)

    Return tuple of zero or more clef marks.
    '''
    from abjad.tools import contexttools

    return contexttools.get_context_marks_attached_to_component(component, klasses=(contexttools.ClefMark,))



'''Abjad ``mathtools`` package.

    Dependencies:

    The ``mathtools`` package imports no other Abjad modules.
    Do not add modules to mathtools with intrapackage imports.

'''
from abjad.tools import importtools

importtools.import_structured_package(__path__[0], globals(), package_root_name='abjad')

_documentation_section = 'core'



from abjad.tools.abctools import AbjadObject
from abjad.tools.configurationtools.get_lilypond_version_string import get_lilypond_version_string


class LilyPondLanguageToken(AbjadObject):
    r'''.. versionadded:: 2.0

    LilyPond language token::

        >>> lilypondfiletools.LilyPondLanguageToken()
        LilyPondLanguageToken('english')

    Return LilyPond language token.

    .. versionchanged:: 2.9
        format with LilyPond ``\language`` command instead of LilyPond ``\include`` command.
    '''

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __repr__(self):
        from abjad import ABJCFG
        return '{}({!r})'.format(
            self._class_name, ABJCFG['lilypond_language'])

    ### PUBLIC PROPERTIES ###

    @property
    def lilypond_format(self):
        r'''Format contribution of LilyPond language token::

            >>> lilypondfiletools.LilyPondLanguageToken().lilypond_format
            '\\language "english"'

        Return string.
        '''
        from abjad import ABJCFG
        lilypond_language = ABJCFG['lilypond_language']
        return r'\language "%s"' % lilypond_language.lower()



def is_nonnegative_integer_equivalent_number(expr):
    '''.. versionadded:: 2.0

    True when `expr` is a nonnegative integer-equivalent number. Otherwise false::

        >>> mathtools.is_nonnegative_integer_equivalent_number(Duration(4, 2))
        True

    Return boolean.
    '''
    from abjad.tools import mathtools

    return mathtools.is_integer_equivalent_number(expr) and 0 <= expr



from abjad.tools import durationtools


def shift_aggregate_offset_by_rational(intervals, rational):
    '''Shift the aggregate offset of `intervals` by `rational` ::

        >>> from abjad.tools.timeintervaltools import TimeInterval
        >>> from abjad.tools.timeintervaltools import TimeIntervalTree

    ::

        >>> a = TimeInterval(-1, 3)
        >>> b = TimeInterval(6, 12)
        >>> c = TimeInterval(9, 16)
        >>> tree = TimeIntervalTree([a, b, c])
        >>> timeintervaltools.shift_aggregate_offset_by_rational(tree, Fraction(1, 3))
        TimeIntervalTree([
            TimeInterval(Offset(-2, 3), Offset(10, 3), {}),
            TimeInterval(Offset(19, 3), Offset(37, 3), {}),
            TimeInterval(Offset(28, 3), Offset(49, 3), {})
        ])

    Return TimeIntervalTree.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    rational = durationtools.Duration(rational)

    if not tree or rational == 0:
        return tree

    return timeintervaltools.TimeIntervalTree([
        x.shift_by_rational(rational) for x in tree
    ])



import copy
import itertools


def replace_leaves_in_expr_with_parallel_voices(expr):
    r'''Replace leaves in `expr` with two parallel voices containing copies of
    leaves in `expr`:

    ::

        >>> c = p('{ c c c c }')
        >>> f(c)
        {
            c4
            c4
            c4
            c4
        }

    ::

        >>> leaftools.replace_leaves_in_expr_with_parallel_voices(c.leaves[1:3])
        ([Note('c4'), Note('c4')], [Note('c4'), Note('c4')])

    ::

        >>> f(c)
        {
            c4
            <<
                \new Voice {
                    c4
                    c4
                }
                \new Voice {
                    c4
                    c4
                }
            >>
            c4
        }

    If leaves in `expr` have different immediate parents, parallel voices will
    be created in each parent:

    ::

        >>> c = p(r'{ c8 \times 2/3 { c8 c c } \times 4/5 { c16 c c c c } c8 }')
        >>> f(c)
        {
            c8
            \times 2/3 {
                c8
                c8
                c8
            }
            \times 4/5 {
                c16
                c16
                c16
                c16
                c16
            }
            c8
        }

    ::

        >>> leaves = leaftools.replace_leaves_in_expr_with_parallel_voices(c.leaves[2:7])

    ::

        >>> f(c)
        {
            c8
            \times 2/3 {
                c8
                <<
                    \new Voice {
                        c8
                        c8
                    }
                    \new Voice {
                        c8
                        c8
                    }
                >>
            }
            \times 4/5 {
                <<
                    \new Voice {
                        c16
                        c16
                        c16
                    }
                    \new Voice {
                        c16
                        c16
                        c16
                    }
                >>
                c16
                c16
            }
            c8
        }

    Returns a list leaves in upper voice, and a list of leaves in lower voice.
    '''    
    from abjad.tools import containertools
    from abjad.tools import iterationtools
    from abjad.tools import voicetools

    leaves = [leaf for leaf in iterationtools.iterate_leaves_in_expr(expr)]

    upper_leaves = []
    lower_leaves = []

    for parent, group in itertools.groupby(leaves, lambda x: x._parent):
        grouped_leaves = list(group)
        start_idx = parent.index(grouped_leaves[0])
        stop_idx = parent.index(grouped_leaves[-1])

        container = containertools.Container()
        container.is_parallel = True
        upper_voice = voicetools.Voice(copy.deepcopy(grouped_leaves))
        lower_voice = voicetools.Voice(copy.deepcopy(grouped_leaves))
        container.extend([upper_voice, lower_voice])

        upper_leaves.extend(upper_voice[:])
        lower_leaves.extend(lower_voice[:])

        parent[start_idx:stop_idx+1] = [container]
        
    return upper_leaves, lower_leaves



from abjad import *
import py.test


def test_componenttools_get_component_in_expr_with_name_01():

    staff = Staff(r"\new Voice { c'8 d'8 } \new Voice { e'8 f'8 } \new Voice { g'4 }")
    staff[0].name = 'top voice'
    staff[1].name = 'middle voice'
    staff[2].name = 'bottom voice'

    assert componenttools.get_component_in_expr_with_name(staff, 'top voice') == staff[0]
    assert componenttools.get_component_in_expr_with_name(staff, 'middle voice') == staff[1]
    assert componenttools.get_component_in_expr_with_name(staff, 'bottom voice') == staff[2]


def test_componenttools_get_component_in_expr_with_name_02():

    staff = Staff(r"\new Voice { c'8 d'8 } \new Voice { e'8 f'8 } \new Voice { g'4 }")
    staff[0].name = 'outer voice'
    staff[1].name = 'middle voice'
    staff[2].name = 'outer voice'

    py.test.raises(ExtraNamedComponentError,
        "componenttools.get_component_in_expr_with_name(staff, 'outer voice')")
    py.test.raises(MissingNamedComponentError,
        "componenttools.get_component_in_expr_with_name(staff, 'no voice')")



from abjad.tools import durationtools


def calculate_depth_centroid_of_intervals_in_interval(intervals, interval):
    '''Return the weighted mean of the depth tree of `intervals` in `interval`,
    such that the centroids of each interval of the depth tree are the values,
    and the weights are the depths at each interval of the depth tree.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    depth = timeintervaltools.compute_depth_of_intervals_in_interval(tree, interval)
    weighted_centroids = sum([x.center * x['depth'] for x in depth])
    sum_of_weights = sum([x['depth'] for x in depth])

    if not sum_of_weights:
        return None

    return durationtools.Offset(weighted_centroids) / sum_of_weights



from abjad.tools.abctools.ImmutableAbjadObject import ImmutableAbjadObject


class NonreducedRatio(ImmutableAbjadObject, tuple):
    '''.. versionadded:: 2.11

    Nonreduced ratio of one or more nonzero integers.

    Initialize from one or more nonzero integers::

        >>> mathtools.NonreducedRatio(2, 4, 2)
        NonreducedRatio(2, 4, 2)

    Or initialize from a tuple or list::

        >>> ratio = mathtools.NonreducedRatio((2, 4, 2))
        >>> ratio
        NonreducedRatio(2, 4, 2)

    Use a tuple to return ratio integers.

        >>> tuple(ratio)
        (2, 4, 2)

    Nonreduced ratios are immutable.
    '''

    ### INITIALIZER ###

    def __new__(klass, *args):
        if len(args) == 1 and isinstance(args[0], (list, tuple)):
            args = args[0]
        assert args, repr(args)
        assert all([x != 0 for x in args]), repr(args) 
        self = tuple.__new__(klass, args)
        return self

    ### SPECIAL METHODS ###

    def __eq__(self, other):
        other = type(self)(other)
        return tuple(self) == tuple(other)
        
    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _positional_argument_values(self):
        return tuple(self)

    ### PRIVATE METHODS ###

    # do not indent in storage
    def _get_tools_package_qualified_repr_pieces(self, is_indented=True):
        return [''.join(
            ImmutableAbjadObject._get_tools_package_qualified_repr_pieces(self, is_indented=False))]



from abjad import *


def test_Container___init___01():
    '''Init empty container.
    '''

    container = Container([])

    r'''
    {
    }
    '''

    assert isinstance(container, Container)
    assert container.lilypond_format == '{\n}'


def test_Container___init___02():
    '''Init container with LilyPond note-entry string.
    '''

    container = Container("c'8 d'8 e'8")

    r'''
    {
        c'8
        d'8
        e'8
    }
    '''

    assert isinstance(container, Container)
    assert container.lilypond_format == "{\n\tc'8\n\td'8\n\te'8\n}"


def test_Container___init___03():
    '''Init container with RTM-syntax string.
    '''

    container = Container('rtm: (1 (1 1 1)) (2 (2 (1 (1 1 1)) 2))')

    r'''
    {
        \times 2/3 {
            c'8
            c'8
            c'8
        }
        \times 4/5 {
            c'4
            \times 2/3 {
                c'16
                c'16
                c'16
            }
            c'4
        }
    }
    '''

    assert isinstance(container, Container)
    assert container.lilypond_format == "{\n\t\\times 2/3 {\n\t\tc'8\n\t\tc'8\n\t\tc'8\n\t}\n\t\\times 4/5 {\n\t\tc'4\n\t\t\\times 2/3 {\n\t\t\tc'16\n\t\t\tc'16\n\t\t\tc'16\n\t\t}\n\t\tc'4\n\t}\n}"

def test_Container___init___04():
    '''Init container with "reduced ly" syntax string.
    '''

    container = Container('abj: 2/3 { 8 8 8 }')

    r'''
    \times 2/3 {
        c'8
        c'8
        c'8
    }
    '''

    assert isinstance(container, Container)
    assert container.lilypond_format == "{\n\t\\times 2/3 {\n\t\tc'8\n\t\tc'8\n\t\tc'8\n\t}\n}"



import copy
from abjad.tools import contexttools
from abjad.tools import durationtools
from abjad.tools import formattools
from abjad.tools import mathtools
from abjad.tools.containertools.FixedDurationContainer import FixedDurationContainer


class Measure(FixedDurationContainer):
    r'''.. versionadded:: 1.1

    .. versionchanged:: 2.9 measure now inherits from fixed-duration container.

    Abjad model of a measure::

        >>> measure = Measure((4, 8), "c'8 d' e' f'")

    ::

        >>> measure
        Measure(4/8, [c'8, d'8, e'8, f'8])

    ::

        >>> f(measure)
        {
            \time 4/8
            c'8
            d'8
            e'8
            f'8
        }

    Return measure object.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_always_format_time_signature', '_automatically_adjust_time_signature', 
        '_measure_number', )

    ### INITIALIZER ###

    def __init__(self, time_signature, music=None, **kwargs):
        # set time signature adjustment indicator before contents initialization
        self._automatically_adjust_time_signature = False
        FixedDurationContainer.__init__(self, time_signature, music)
        self._always_format_time_signature = False
        self._measure_number = None
        time_signature = contexttools.TimeSignatureMark(time_signature)
        time_signature.attach(self)
        self._initialize_keyword_values(**kwargs)

    ### SPECIAL METHODS ###

    def __add__(self, arg):
        '''Add two measures together in-score or outside-of-score.
        Wrapper around measuretools.fuse_measures.
        '''
        assert isinstance(arg, type(self))
        from abjad.tools import measuretools
        new = measuretools.fuse_measures([self, arg])
        return new

    def __delitem__(self, i):
        '''Container item deletion with optional time signature adjustment.
        '''
        old_time_signature = contexttools.get_effective_time_signature(self)
        old_denominator = getattr(old_time_signature, 'denominator', None)
        FixedDurationContainer.__delitem__(self, i)
        self._conditionally_adjust_time_signature(old_denominator)

    def __getnewargs__(self):
        time_signature = contexttools.get_effective_time_signature(self)
        return (time_signature.pair, )

    def __repr__(self):
        '''String form of measure with parentheses for interpreter display.
        '''
        class_name = type(self).__name__
        forced_time_signature = contexttools.get_time_signature_mark_attached_to_component(self)
        summary = self._summary
        length = len(self)
        if forced_time_signature and length:
            return '%s(%s, [%s])' % (class_name, forced_time_signature, summary)
        elif forced_time_signature:
            return '%s(%s)' % (class_name, forced_time_signature)
        elif length:
            return '%s([%s])' % (class_name, summary)
        else:
            return '%s()' % class_name

    def __setitem__(self, i, expr):
        '''Container setitem logic with optional time signature adjustment.

        .. versionchanged:: 2.9
            Measure setitem logic now adjusts time signatue automatically
            when ``adjust_time_signature_automatically`` is true.
        '''
        old_time_signature = contexttools.get_effective_time_signature(self)
        old_denominator = getattr(old_time_signature, 'denominator', None)
        FixedDurationContainer.__setitem__(self, i, expr)
        self._conditionally_adjust_time_signature(old_denominator)

    def __str__(self):
        '''String form of measure with pipes for single string display.
        '''
        forced_time_signature = contexttools.get_effective_time_signature(self)
        summary = self._summary
        length = len(self)
        if forced_time_signature and length:
            return '|%s, %s|' % (forced_time_signature, summary)
        elif forced_time_signature:
            return '|%s|' % forced_time_signature
        elif length:
            return '|%s|' % summary
        else:
            return '| |'

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _compact_representation(self):
        '''Display form of measure used for spanners to display
        potentially many spanned measures one after the other.
        '''
        return '|%s(%s)|' % (contexttools.get_effective_time_signature(self), len(self))

    # essentially the same as container version of method;
    # the definition given here adds one line to remove
    # time signature immediately after instantiation
    # because the mark-copying code will then provide time signature.
    def _copy_with_marks_but_without_children_or_spanners(self):
        from abjad.tools import marktools
        new = type(self)(*self.__getnewargs__())
        # only the following line differs from Conatainer
        contexttools.detach_time_signature_marks_attached_to_component(new)
        if getattr(self, '_override', None) is not None:
            new._override = copy.copy(self.override)
        if getattr(self, '_set', None) is not None:
            new._set = copy.copy(self.set)
        for mark in marktools.get_marks_attached_to_component(self):
            new_mark = copy.copy(mark)
            new_mark.attach(new)
        new.is_parallel = self.is_parallel
        return new

    ### PRIVATE METHODS ###

    def _check_duration(self):
        from abjad.tools import contexttools
        effective_time_signature = contexttools.get_effective_time_signature(self)
        if effective_time_signature.has_non_power_of_two_denominator and effective_time_signature.suppress:
            raise Exception('Can not suppress time signature with non-power of two denominator.')
        if effective_time_signature.duration < self.preprolated_duration:
            raise OverfullContainerError
        if self.preprolated_duration < effective_time_signature.duration:
            raise UnderfullContainerError

    def _conditionally_adjust_time_signature(self, old_denominator):
        if self.automatically_adjust_time_signature:
            naive_time_signature = self.preprolated_duration
            better_time_signature = mathtools.NonreducedFraction(naive_time_signature)
            better_time_signature = better_time_signature.with_denominator(old_denominator)
            better_time_signature = contexttools.TimeSignatureMark(better_time_signature)
            contexttools.detach_time_signature_marks_attached_to_component(self)
            better_time_signature.attach(self)

    def _format_content_pieces(self):
        result = []
        # the class name test here functions to exclude scaleDurations from anonymous and dynamic measures
        # TODO: subclass this prooperly on anonymous and dynamic measures
        if self.has_non_power_of_two_denominator and self._class_name == 'Measure':
            result.append("\t\\scaleDurations #'(%s . %s) {" % (
                self.implied_prolation.numerator,
                self.implied_prolation.denominator))
            result.extend( ['\t' + x for x in FixedDurationContainer._format_content_pieces(self)])
            result.append('\t}')
        else:
            result.extend(FixedDurationContainer._format_content_pieces(self))
        return result

    def _format_opening_slot(self, format_contributions):
        r'''This is the slot where LilyPond grob \override commands live.
        This is also the slot where LilyPond \time commands live.
        '''
        result = []
        result.append(('comments', format_contributions.get('opening', {}).get('comments', [])))
        result.append(('grob overrides', format_contributions.get('grob overrides', [])))
        result.append(('context settings', format_contributions.get('context settings', [])))
        result.append(('context marks', format_contributions.get('opening', {}).get('context marks', [])))
        #result.append(formattools.get_comment_format_contributions_for_slot(self, 'opening'))
        #result.append(formattools.get_grob_override_format_contributions(self))
        #result.append(formattools.get_context_setting_format_contributions(self))
        #result.append(formattools.get_context_mark_format_contributions_for_slot(self, 'opening'))
        return self._format_slot_contributions_with_indent(result)

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def has_non_power_of_two_denominator(self):
        '''True when measure time signature denominator is not an integer power of 2::

            >>> measure = Measure((5, 9), "c'8 d' e' f' g'")
            >>> measure.has_non_power_of_two_denominator
            True
    
        Otherwise false::

            >>> measure = Measure((5, 8), "c'8 d' e' f' g'")
            >>> measure.has_non_power_of_two_denominator
            False

        Return boolean.
        '''
        return contexttools.get_effective_time_signature(self).has_non_power_of_two_denominator

    @property
    def has_power_of_two_denominator(self):
        '''True when measure time signature denominator is an integer power of 2::

            >>> measure = Measure((5, 8), "c'8 d' e' f' g'")
            >>> measure.has_power_of_two_denominator
            True

        Otherwise false::

            >>> measure = Measure((5, 9), "c'8 d' e' f' g'")
            >>> measure.has_power_of_two_denominator
            False

        Return boolean.
        '''
        return not self.has_non_power_of_two_denominator

    @property
    def implied_prolation(self):
        '''Implied prolation of measure time signature::

            >>> measure = Measure((5, 12), "c'8 d' e' f' g'")

        ::

            >>> measure.implied_prolation
            Multiplier(2, 3)

        Return multiplier.
        '''
        return contexttools.get_effective_time_signature(self).implied_prolation

    @property
    def is_full(self):
        '''True when prolated duration equals time signature duration::

            >>> measure = Measure((4, 8), "c'8 d'8 e'8 f'8")

        ::

            >>> measure.is_full
            True

        Otherwise false.

        Return boolean.
        '''
        return FixedDurationContainer.is_full.fget(self)

    @property
    def is_misfilled(self):
        '''.. versionadded:: 2.9

        True when measure is either underfull or overfull::

            >>> measure = Measure((3, 4), "c' d' e' f'")

        ::

            >>> measure
            Measure(3/4, [c'4, d'4, e'4, f'4])

        ::

            >>> measure.is_misfilled
            True

        Otherwise false::

            >>> measure = Measure((3, 4), "c' d' e'")

        ::

            >>> measure
            Measure(3/4, [c'4, d'4, e'4])

        ::

            >>> measure.is_misfilled
            False

        Return boolean.
        '''
        return FixedDurationContainer.is_overfull.fget(self)

    @property
    def is_overfull(self):
        '''.. versionadded:: 1.1

        True when prolated duration is greater than time signature duration::

            >>> measure = Measure((3, 4), "c'4 d' e' f'")

        ::

            >>> measure.is_overfull
            True

        Otherwise false.

        Return boolean.
        '''
        return FixedDurationContainer.is_overfull.fget(self)

    @property
    def is_underfull(self):
        '''.. versionadded:: 1.1

        True when prolated duration is less than time signature duration::

            >>> measure = Measure((3, 4), "c'4 d'")

        ::

            >>> measure.is_underfull
            True

        Otherwise false.

        Return boolean.
        '''
        return FixedDurationContainer.is_underfull.fget(self)

    @property
    def lilypond_format(self):
        self._check_duration()
        return self._format_component()

    @property
    def measure_number(self):
        r'''1-indexed measure number::

            >>> staff = Staff()
            >>> staff.append(Measure((3, 4), "c' d' e'"))
            >>> staff.append(Measure((2, 4), "f' g'"))

        ::

            >>> f(staff)
            \new Staff {
                {
                    \time 3/4
                    c'4
                    d'4
                    e'4
                }
                {
                    \time 2/4
                    f'4
                    g'4
                }
            }

        ::

            >>> staff[0].measure_number
            1

        ::

            >>> staff[1].measure_number
            2

        Return positive integer.
        '''
        self._update_prolated_offset_values_of_entire_score_tree_if_necessary()
        return self._measure_number

    @property
    def preprolated_duration(self):
        '''Preprolated duration of measure::

            >>> measure = Measure((5, 12), "c'8 d' e' f' g'")

        ::

            >>> measure.preprolated_duration
            Duration(5, 12)

        Equal measure contents duration times time signature multiplier.

        Return duration.
        '''
        from abjad.tools import contexttools
        return contexttools.get_effective_time_signature(self).implied_prolation * self.contents_duration

    ### READ / WRITE PUBLIC PROPERTIES ###

    @apply
    def always_format_time_signature():
        def fget(self):
            '''.. versionadded:: 2.9
        
            Read / write flag to indicate whether time signature
            should appear in LilyPond format even when not expected.

            Set to true when necessary to print the same signature repeatedly.

            Default to false.

            Return boolean.
            '''
            return self._always_format_time_signature
        def fset(self, expr):
            assert isinstance(expr, bool)
            self._always_format_time_signature = expr
        return property(**locals())

    @apply
    def automatically_adjust_time_signature():
        def fget(self):
            '''.. versionadded:: 2.9

            Read / write flag to indicate whether time signature
            should update automatically following contents-changing operations::

                >>> measure = Measure((3, 4), "c' d' e'")

            ::

                >>> measure
                Measure(3/4, [c'4, d'4, e'4])

            ::

                >>> measure.automatically_adjust_time_signature = True
                >>> measure.append('r')

            ::

                >>> measure
                Measure(4/4, [c'4, d'4, e'4, r4])
            
            Default to false.

            Return boolean.
            '''
            return self._automatically_adjust_time_signature
        def fset(self, expr):
            assert isinstance(expr, bool)
            self._automatically_adjust_time_signature = expr
        return property(**locals()) 

    @property
    def target_duration(self):
        r'''.. versionadded:: 2.9

        Read-only target duration of measure always equal to duration of effective time signature.
        '''
        return contexttools.get_effective_time_signature(self).duration



from abjad.tools import durationtools
from abjad.tools import leaftools


def scale_contents_of_tuplets_in_expr_by_multiplier(tuplet, multiplier):
    r'''Scale contents of fixed-duration `tuplet` by `multiplier`::

        >>> tuplet = tuplettools.FixedDurationTuplet((3, 8), "c'8 d'8 e'8 f'8 g'8")

    ::

        >>> f(tuplet)
        \fraction \times 3/5 {
            c'8
            d'8
            e'8
            f'8
            g'8
        }

    ::

        >>> tuplettools.scale_contents_of_tuplets_in_expr_by_multiplier(tuplet, Fraction(2))
        FixedDurationTuplet(3/4, [c'4, d'4, e'4, f'4, g'4])

    ::

        >>> f(tuplet)
        \fraction \times 3/5 {
            c'4
            d'4
            e'4
            f'4
            g'4
        }

    Preserve `tuplet` multiplier.

    Return tuplet.
    '''
    from abjad.tools import tuplettools

    # check input
    multiplier = durationtools.Multiplier(multiplier)
    if not isinstance(tuplet, tuplettools.FixedDurationTuplet):
        raise TypeError('must be fixed-duration tuplet.')

    # find new target duration
    old_target_duration = tuplet.target_duration
    new_target_duration = multiplier * old_target_duration

    # change tuplet target duration
    tuplet.target_duration = new_target_duration

    # if multiplier is note head assignable, scale contents graphically
    if multiplier.is_assignable:
        for component in tuplet[:]:
            if isinstance(component, leaftools.Leaf):
                leaftools.scale_preprolated_leaf_duration(component, multiplier)

    # otherwise doctor up tuplet multiplier, if necessary
    elif not tuplet.multiplier.is_proper_tuplet_multiplier:
        tuplettools.fix_contents_of_tuplets_in_expr(tuplet)

    # return tuplet
    return tuplet



from abjad.tools import componenttools
from abjad.tools import leaftools


def iterate_timeline_in_expr(expr, klass=None, reverse=False):
    r'''.. versionadded:: 2.10

    Iterate timeline forward in `expr`::

        >>> score = Score([])
        >>> score.append(Staff(notetools.make_repeated_notes(4, Duration(1, 4))))
        >>> score.append(Staff(notetools.make_repeated_notes(4)))
        >>> pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(score)

    ::

        >>> f(score)
        \new Score <<
            \new Staff {
                c'4
                d'4
                e'4
                f'4
            }
            \new Staff {
                g'8
                a'8
                b'8
                c''8
            }
        >>

    ::

        >>> for leaf in iterationtools.iterate_timeline_in_expr(score):
        ...     leaf
        ...
        Note("c'4")
        Note("g'8")
        Note("a'8")
        Note("d'4")
        Note("b'8")
        Note("c''8")
        Note("e'4")
        Note("f'4")

    Iterate timeline backward in `expr`::

    ::

        >>> for leaf in iterationtools.iterate_timeline_in_expr(score, reverse=True):
        ...     leaf
        ...
        Note("f'4")
        Note("e'4")
        Note("d'4")
        Note("c''8")
        Note("b'8")
        Note("c'4")
        Note("a'8")
        Note("g'8")

    Iterate leaves when `klass` is none.

    .. todo:: optimize to avoid behind-the-scenes full-score traversal.
    '''
    from abjad.tools import componenttools
    from abjad.tools import iterationtools
    from abjad.tools import leaftools

    if klass is None:
        klass = leaftools.Leaf

    component_generator = iterationtools.iterate_components_in_expr(expr, klass=klass)
    components = list(component_generator)

    def _backward_sort_helper(component_1, component_2):
        result = cmp(component_1.stop_offset, component_2.stop_offset)
        if result == 0:
            return cmp(component_1.parentage.score_index, component_2.parentage.score_index)
        else:   
            # note negative result of cmp() is returned
            # for backward time sort
            return -result

    def _forward_sort_helper(component_1, component_2):
        result = cmp(component_1.start_offset, component_2.start_offset)
        if result == 0:
            return cmp(component_1.parentage.score_index, component_2.parentage.score_index)
        else:
            return result

    if not reverse:
        components.sort(_forward_sort_helper)
    else:
        components.sort(_backward_sort_helper)

    for component in components:
        yield component



# TODO: make public and bind to some class somewhere
def _format_note_head(note_head):
    from abjad.tools.lilypondfiletools._format_lilypond_attribute import _format_lilypond_attribute
    from abjad.tools.lilypondfiletools._format_lilypond_value import _format_lilypond_value
    from abjad.tools import chordtools

    # make sure note head has pitch
    assert note_head.written_pitch
    result = []

    # format chord note head with optional tweaks
    if isinstance(note_head._client, chordtools.Chord):
        for key, value in vars(note_head.tweak).iteritems():
            if not key.startswith('_'):
                result.append(r'\tweak %s %s' % (
                    _format_lilypond_attribute(key),
                    _format_lilypond_value(value)))

    # format note head pitch
    kernel = note_head.written_pitch.lilypond_format
    if note_head.is_forced:
        kernel += '!'
    if note_head.is_cautionary:
        kernel += '?'
    result.append(kernel)
    result = '\n'.join(result)

    # return formatted note head
    return result



from abjad import *


def test_TaleaRhythmMaker_tie_split_notes_01():

    maker = rhythmmakertools.TaleaRhythmMaker([5], 16, tie_split_notes=True)
    divisions = [(2, 8), (2, 8), (2, 8), (2, 8)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    measures = measuretools.make_measures_with_full_measure_spacer_skips(divisions)
    staff = Staff(measures)
    measures = measuretools.replace_contents_of_measures_in_expr(staff, music)

    r'''
    \new Staff {
        {
            \time 2/8
            c'4 ~
        }
        {
            c'16
            c'8. ~
        }
        {
            c'8
            c'8 ~
        }
        {
            c'8.
            c'16
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 2/8\n\t\tc'4 ~\n\t}\n\t{\n\t\tc'16\n\t\tc'8. ~\n\t}\n\t{\n\t\tc'8\n\t\tc'8 ~\n\t}\n\t{\n\t\tc'8.\n\t\tc'16\n\t}\n}"


def test_TaleaRhythmMaker_tie_split_notes_02():

    maker = rhythmmakertools.TaleaRhythmMaker([5], 16, tie_split_notes=True)
    divisions = [(3, 16), (5, 8), (4, 8), (7, 16)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    measures = measuretools.make_measures_with_full_measure_spacer_skips(divisions)
    staff = Staff(measures)
    measures = measuretools.replace_contents_of_measures_in_expr(staff, music)

    r'''
    \new Staff {
        {
            \time 3/16
            c'8. ~
        }
        {
            \time 5/8
            c'8
            c'4 ~
            c'16
            c'8. ~
        }
        {
            \time 4/8
            c'8
            c'4 ~
            c'16
            c'16 ~
        }
        {
            \time 7/16
            c'4
            c'8.
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 3/16\n\t\tc'8. ~\n\t}\n\t{\n\t\t\\time 5/8\n\t\tc'8\n\t\tc'4 ~\n\t\tc'16\n\t\tc'8. ~\n\t}\n\t{\n\t\t\\time 4/8\n\t\tc'8\n\t\tc'4 ~\n\t\tc'16\n\t\tc'16 ~\n\t}\n\t{\n\t\t\\time 7/16\n\t\tc'4\n\t\tc'8.\n\t}\n}"



from abjad.tools import componenttools
from abjad.tools import durationtools


# TODO: optimize without full-component traversal.
def get_vertical_moment_at_offset_in_expr(expr, offset):
    r'''.. versionadded:: 2.0

    Get vertical moment at `offset` in `expr`::

        >>> score = Score([])
        >>> staff = Staff(r"\times 4/3 { d''8 c''8 b'8 }")
        >>> score.append(staff)

    ::

        >>> piano_staff = scoretools.PianoStaff([])
        >>> piano_staff.append(Staff("a'4 g'4"))
        >>> piano_staff.append(Staff(r"""\clef "bass" f'8 e'8 d'8 c'8"""))
        >>> score.append(piano_staff)

    ::

        >>> f(score)
        \new Score <<
            \new Staff {
                \fraction \times 4/3 {
                    d''8
                    c''8
                    b'8
                }
            }
            \new PianoStaff <<
                \new Staff {
                    a'4
                    g'4
                }
                \new Staff {
                    \clef "bass"
                    f'8
                    e'8
                    d'8
                    c'8
                }
            >>
        >>

    ::

        >>> args = (piano_staff, durationtools.Offset(1, 8))

    ::

        >>> verticalitytools.get_vertical_moment_at_offset_in_expr(*args)
        VerticalMoment(1/8, <<2>>)

    ::

        >>> vertical_moment = _
        >>> vertical_moment.leaves
        (Note("a'4"), Note("e'8"))
    
    Return vertical moment.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import verticalitytools

    def find_index(container, offset):
        '''Based off of Python's bisect.bisect() function.'''
        lo = 0
        hi = len(container)
        while lo < hi:
            mid = (lo + hi) // 2
            start_offset = container[mid].start_offset
            stop_offset = container[mid].stop_offset
            if start_offset <= offset < stop_offset:
                lo = mid + 1
            elif start_offset < stop_offset: # if container[mid] is of non-zero duration
                hi = mid
            else: # else, container[mid] _is_ of zero duration, so we skip it
                lo = mid + 1
        return lo - 1

    def recurse(component, offset):
        result = []
        if component.start_offset <= offset < component.stop_offset:
            result.append(component)
            if hasattr(component, '_music'):
                if component.is_parallel:
                    for x in component:
                        result.extend(recurse(x, offset))
                else:
                    child = component[find_index(component, offset)]
                    result.extend(recurse(child, offset))
        return result

    offset = durationtools.Offset(offset)

    governors = []
    message = 'must be Abjad component or list or tuple of Abjad components.'
    if isinstance(expr, componenttools.Component):
        governors.append(expr)
    elif isinstance(expr, (list, tuple)):
        for x in expr:
            if isinstance(x, componenttools.Component):
                governors.append(x)
            else:
                raise TypeError(message)
    else:
        raise TypeError(message)
    governors.sort(lambda x, y: cmp(x.parentage.score_index, y.parentage.score_index))
    governors = tuple(governors)

    components = []
    for governor in governors:
        components.extend(recurse(governor, offset))
    components.sort(lambda x, y: cmp(x.parentage.score_index, y.parentage.score_index))
    components = tuple(components)

    vertical_moment = verticalitytools.VerticalMoment(offset, governors, components)

    return vertical_moment



def detach_dynamic_marks_attached_to_component(component):
    r'''.. versionadded:: 2.3

    Detach dynamic marks attached to `component`::

        >>> staff = Staff("c'4 d'4 e'4 f'4")
        >>> dynamic_mark = contexttools.DynamicMark('p')
        >>> dynamic_mark.attach(staff[0])
        DynamicMark('p')(c'4)

    ::

        >>> f(staff)
        \new Staff {
            c'4 \p
            d'4
            e'4
            f'4
        }

    ::

        >>> contexttools.detach_dynamic_marks_attached_to_component(staff[0])
        (DynamicMark('p'),)

    ::

        >>> f(staff)
        \new Staff {
            c'4
            d'4
            e'4
            f'4
        }

    Return tuple of zero or more dynamic marks.
    '''
    from abjad.tools import contexttools

    marks = []
    for mark in contexttools.get_dynamic_marks_attached_to_component(component):
        mark.detach()
        marks.append(mark)

    return tuple(marks)



def is_formattable_context_mark_for_component(mark, component):
    '''Return True if ContextMark `mark` can format for `component`.''' 

    from abjad.tools import componenttools
    from abjad.tools import contexttools
    from abjad.tools import measuretools

    if mark.start_component is None:
        return False

    if isinstance(mark.start_component, measuretools.Measure):
        if mark.start_component is component:
            if not isinstance(mark, contexttools.TimeSignatureMark):
                return True
            elif component.always_format_time_signature:
                return True
            else:
                previous_measure = measuretools.get_previous_measure_from_component(
                    mark.start_component)
                if previous_measure is not None:
                    previous_effective_time_signature = \
                        contexttools.get_effective_time_signature(previous_measure)
                else:
                    previous_effective_time_signature = None
                if not mark == previous_effective_time_signature:
                    return True

    elif mark._format_slot == 'right':
        if mark.start_component is component:
            return True

    elif mark.start_component is component:
        return True

    else:
        improper_parentage = componenttools.get_improper_parentage_of_component(component)
        if mark.effective_context in improper_parentage:
            proper_parentage = componenttools.get_proper_parentage_of_component(component)
            if mark.effective_context not in proper_parentage:
                if mark.start_component.start == component.start:
                    return True

    return False



from abjad import *


def test_tuplettools_change_tuplets_in_expr_to_fixed_duration_tuplets_01():

    staff = Staff(r"\times 2/3 { c'8 d'8 e'8 } \times 2/3 { c'8 d'8 e'8 }") 
    assert all([x._class_name == 'Tuplet' for x in staff])

    tuplettools.change_tuplets_in_expr_to_fixed_duration_tuplets(staff)
    assert all([isinstance(x, tuplettools.FixedDurationTuplet) for x in staff])
